/**      DisplayBuffer.h
 * (nearly) compatible to original dogm-graphic.h, enhanced with
 * flip_buffer and back reading functions. It allows nice rendering functions 
 * and easy buffer access.
 **/

#ifndef _DISPLAYBUFFER_
#define _DISPLAYBUFFER_

#include "Prometheus.h"

extern volatile uint8_t buf_current_page;
extern volatile uint8_t buf_current_column;

extern uint8_t back_buffer[LCD_RAM_PAGES][LCD_WIDTH];
extern uint8_t front_buffer[LCD_RAM_PAGES][LCD_WIDTH];

// New stuff

#define ANIM_STOP        0 //Do nothing
#define ANIM_FLIP        1 //Just replace content
#define ANIM_PUSH_LEFT   2 //Insert new content from right side
#define ANIM_PUSH_RIGHT  3 //Insert new content from left side

void flip_buffer(uint8_t animation);

//The buffer engine needs to be called frequently
void buffer_engine(void);


// Legecy stuff

//initializes the display in 6 o'clock mode & 4x booster for 2.4-3.3V supply voltage
//for EA-DOGM132 and EA_DOGM128
//void buf_init(void);

//write data word or command to the LCD
void buf_data     (uint8_t data);
void buf_command  (uint8_t cmd);

//User functions to write raw data from flash
void buf_put_raw     (PGM_P data, uint8_t length);
//Functions to draw images from progmem, added by Oliver S.
void buf_draw_image  (const uint8_t *progmem_image, uint8_t pages, uint8_t columns, int8_t inverse);
void buf_draw_image_xy(const uint8_t *progmem_image, uint8_t x, uint8_t y, uint8_t pages, uint8_t columns, int8_t inverse);

//Move cursor
void buf_moveto_xy  (uint8_t page, uint8_t column);
void buf_move_xy    (int8_t pages, int16_t columns);

/**
 * Inverts a given rectangle from a giben starting point and size
 */
void buf_invert(uint8_t start_x, uint8_t start_y, uint8_t width_x, uint8_t width_y);

#endif
