/*
 *      Menu.c
 *
 *      Copyright 2007 Oliver Schwaneberg <oschwa2s@oschwa2s-desktop>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include "Menu.h"




//Different menu states (A Leaf is the last layer, which leads to internal functions and a different rendering)
#define MAINSCREEN -1
#define MENU_MAIN 0
#define MENU_OPTIONS 1
#define MENU_PROFILES 2
#define MENU_PROFILES_LEAF 20
#define MENU_PROFILES_TARGET 21
#define MENU_CREDITS 3
//#define MENU_CREDITS_LEAF 30 - not used at all
#define MENU_LANGUAGE 4
#define MENU_CLOCK 5
#define MENU_SENSORS 6
#define MENU_DISPLAY 7
#define MENU_MAINSCREEN   8
#define MENU_SYSTEM 10
#define MENU_DATA_1 11
#define MENU_DATA_2 12
#define MENU_DATA_3 13
#define MENU_DATA_4 14

//Realtime triggered
#define MENU_DEMO 81
#define MENU_SYSTEM_INFO  82
#define MENU_BRIGHTNESS 83

//Standalone applications
#define MENU_CLOCK_EDITOR 101
#define MENU_FILE_BROWSER 102
#define MENU_PH1          103
#define MENU_PH2          104
#define MENU_TEST         105
#define MENU_FLOWMETER_SET 106
#define MENU_FLOWMETER_MIN    107

//For DisplayBuffer
uint8_t anim;

//Defining language settings
int8_t num_of_languages = 2;
const prog_char available_languages_1[] PROGMEM = "Deutsch";
const prog_char available_languages_2[] PROGMEM = "English";
PGM_P available_languages[] = {available_languages_1, available_languages_2};
PGM_P PROGMEM available_languages_P[] = {available_languages_1, available_languages_2};

const prog_char title_menu_languages_GER[] PROGMEM = "Sprachauswahl";
const prog_char title_menu_languages_EN[] PROGMEM = "Choose language";
PGM_P title_menu_languages[] = {title_menu_languages_GER, title_menu_languages_EN};

uint8_t current_language = 0xFF;

//Some initial values
int8_t cur_menu = MAINSCREEN;//Current menu we are in, mainscreen at startup.
uint8_t render_state_changed = 0; // Initial renderState
uint8_t render_redraw = 0; // Initial renderState
int8_t active_index = 0; //entry in menu that is currently active
uint8_t button = BUTTON_NONE;
/** Sub Menu entries **/
const prog_char labels_menu_back_GER [] PROGMEM = "Zurck";
const prog_char labels_menu_back_EN [] PROGMEM = "Back";
PGM_P labels_menu_back[] = {labels_menu_back_GER, labels_menu_back_EN};

//Main menu
const prog_char title_menu_main_GER[] PROGMEM = "Hauptmen";
const prog_char title_menu_main_EN[] PROGMEM = "Main Menu";
PGM_P title_menu_main[] = {title_menu_main_GER, title_menu_main_EN};

int8_t num_labels_menu_main= 6;// MUST match length!
const prog_char labels_menu_main_GER1[] PROGMEM = "Profile";
const prog_char labels_menu_main_GER2[] PROGMEM = "Optionen";
const prog_char labels_menu_main_GER3[] PROGMEM = "Autoren";
const prog_char labels_menu_main_GER4[] PROGMEM = "Demo-Modus";
const prog_char labels_menu_main_GER5[] PROGMEM = "Testmodus";
PGM_P PROGMEM labels_menu_main_GER [] = {labels_menu_main_GER1, labels_menu_main_GER2, labels_menu_main_GER3, labels_menu_main_GER4,labels_menu_main_GER5, labels_menu_back_GER};
const prog_char labels_menu_main_EN1[] PROGMEM = "Profiles";
const prog_char labels_menu_main_EN2[] PROGMEM = "Options";
const prog_char labels_menu_main_EN3[] PROGMEM = "Credits";
const prog_char labels_menu_main_EN4[] PROGMEM = "Demo mode";
const prog_char labels_menu_main_EN5[] PROGMEM = "Test mode";
PGM_P PROGMEM labels_menu_main_EN [] = {labels_menu_main_EN1, labels_menu_main_EN2, labels_menu_main_EN3, labels_menu_main_EN4,labels_menu_main_EN5, labels_menu_back_EN};
PGM_P* labels_menu_main[] = {labels_menu_main_GER, labels_menu_main_EN};

//Profiles sub-menu
//Note: Instead of "User <n>" we must display the stored name from the EEPROM
const prog_char title_menu_profiles_GER [] PROGMEM	= "Profilauswahl";
const prog_char title_menu_profiles_EN [] PROGMEM	= "Choose a profile";
PGM_P title_menu_profiles[] = {title_menu_profiles_GER, title_menu_profiles_EN};
int8_t num_labels_menu_profiles= 9; // MUST match length !
int8_t num_labels_menu_target_profiles= 3;
const prog_char labels_menu_profiles_GER1 [] PROGMEM = "Amazonas";
const prog_char labels_menu_profiles_GER2 [] PROGMEM = "Pazifik";
const prog_char labels_menu_profiles_GER3 [] PROGMEM = "Rhein";
const prog_char labels_menu_profiles_GER4 [] PROGMEM = "Kongo";
const prog_char labels_menu_profiles_GER5 [] PROGMEM = "<1 leer>";
const prog_char labels_menu_profiles_GER6 [] PROGMEM = "<2 leer>";
const prog_char labels_menu_profiles_GER7 [] PROGMEM = "<3 leer>";
const prog_char labels_menu_profiles_GER8 [] PROGMEM = "Durchsuche SD-Karte";
PGM_P PROGMEM labels_menu_profiles_GER [] = {labels_menu_profiles_GER1, labels_menu_profiles_GER2, labels_menu_profiles_GER3,
                                             labels_menu_profiles_GER4, labels_menu_profiles_GER5, labels_menu_profiles_GER6,
                                             labels_menu_profiles_GER7, labels_menu_profiles_GER8, labels_menu_back_GER};
PGM_P PROGMEM labels_menu_target_profiles_GER [] = {labels_menu_profiles_GER5,labels_menu_profiles_GER6,labels_menu_profiles_GER7};
const prog_char labels_menu_profiles_EN1 [] PROGMEM = "Amazon";
const prog_char labels_menu_profiles_EN2 [] PROGMEM = "Pacific";
const prog_char labels_menu_profiles_EN3 [] PROGMEM = "Rhine river";
const prog_char labels_menu_profiles_EN4 [] PROGMEM = "Congo river";
const prog_char labels_menu_profiles_EN5 [] PROGMEM = "<1 empty>";
const prog_char labels_menu_profiles_EN6 [] PROGMEM = "<2 empty>";
const prog_char labels_menu_profiles_EN7 [] PROGMEM = "<3 empty>";
const prog_char labels_menu_profiles_EN8 [] PROGMEM = "Browse SD-Card";
PGM_P PROGMEM labels_menu_profiles_EN [] = {labels_menu_profiles_EN1, labels_menu_profiles_EN2, labels_menu_profiles_EN3,
                                             labels_menu_profiles_EN4, labels_menu_profiles_EN5, labels_menu_profiles_EN6,
                                             labels_menu_profiles_EN7, labels_menu_profiles_EN8, labels_menu_back_EN};
PGM_P PROGMEM labels_menu_target_profiles_EN [] = {labels_menu_profiles_EN5,labels_menu_profiles_EN6,labels_menu_profiles_EN7};
PGM_P* labels_menu_profiles[]= {labels_menu_profiles_GER, labels_menu_profiles_EN};
PGM_P* labels_menu_target_profiles[] = {labels_menu_target_profiles_GER, labels_menu_target_profiles_EN};
const prog_char title_menu_target_profiles_GER [] PROGMEM	= "Whle Speicherstelle";
const prog_char title_menu_target_profiles_EN [] PROGMEM	= "Choose a memory slot";
PGM_P title_menu_target_profiles[] = {title_menu_target_profiles_GER, title_menu_target_profiles_EN};

const prog_char title_menu_options_GER [] PROGMEM	= "Optionen";
const prog_char title_menu_options_EN [] PROGMEM	= "Options";
PGM_P title_menu_options[] = {title_menu_options_GER, title_menu_options_EN};
int8_t num_labels_menu_options= 6; // MUST match length!
const prog_char labels_menu_options_GER1 [] PROGMEM = "Sprache";
const prog_char labels_menu_options_GER2 [] PROGMEM = "Uhr";
const prog_char labels_menu_options_GER3 [] PROGMEM = "Sensoren";
const prog_char labels_menu_options_GER4 [] PROGMEM = "Display";
const prog_char labels_menu_options_5 [] PROGMEM = "System";
PGM_P PROGMEM labels_menu_options_GER [] = {labels_menu_options_GER1, labels_menu_options_GER2, labels_menu_options_GER3,
                                            labels_menu_options_GER4, labels_menu_options_5, labels_menu_back_GER};
const prog_char labels_menu_options_EN1 [] PROGMEM = "Language";
const prog_char labels_menu_options_EN2 [] PROGMEM = "Clock";
const prog_char labels_menu_options_EN3 [] PROGMEM = "Sensors";
const prog_char labels_menu_options_EN4 [] PROGMEM = "Display";
PGM_P PROGMEM labels_menu_options_EN [] = {labels_menu_options_EN1, labels_menu_options_EN2, labels_menu_options_EN3,
                                            labels_menu_options_EN4, labels_menu_options_5, labels_menu_back_EN};
PGM_P* labels_menu_options[] = {labels_menu_options_GER, labels_menu_options_EN};

const prog_char title_menu_clock_GER [] PROGMEM	= "Uhr";
const prog_char title_menu_clock_EN [] PROGMEM	= "Clock";
PGM_P title_menu_clock[] = {title_menu_clock_GER, title_menu_clock_EN};
int8_t num_labels_menu_clock = 6; // MUST match length!
const prog_char labels_menu_clock_GER1 [] PROGMEM = "Uhr einstellen";
const prog_char labels_menu_clock_GER2 [] PROGMEM = "DCF-Funkuhrempfang";
const prog_char labels_menu_clock_GER3 [] PROGMEM = "Autom. Sommerzeit";
const prog_char labels_menu_clock_GER4 [] PROGMEM = "Sommerzeit";
const prog_char labels_menu_clock_GER5 [] PROGMEM = "Mondphasen nutzen";
PGM_P PROGMEM labels_menu_clock_GER [] = {labels_menu_clock_GER1, labels_menu_clock_GER2, labels_menu_clock_GER3,
                                         labels_menu_clock_GER4, labels_menu_clock_GER5, labels_menu_back_GER};
const prog_char labels_menu_clock_EN1 [] PROGMEM = "Set clock";
const prog_char labels_menu_clock_EN2 [] PROGMEM = "DCF radio sync.";
const prog_char labels_menu_clock_EN3 [] PROGMEM = "Auto DST";
const prog_char labels_menu_clock_EN4 [] PROGMEM = "DST";
const prog_char labels_menu_clock_EN5 [] PROGMEM = "Use moon-phases";
PGM_P PROGMEM labels_menu_clock_EN [] = {labels_menu_clock_EN1, labels_menu_clock_EN2, labels_menu_clock_EN3,
                                         labels_menu_clock_EN4, labels_menu_clock_EN5, labels_menu_back_EN};
PGM_P* labels_menu_clock[] = {labels_menu_clock_GER, labels_menu_clock_EN};
                                                       
const prog_char title_menu_sensors_GER [] PROGMEM	= "Sensoren";
const prog_char title_menu_sensors_EN [] PROGMEM	= "Sensors";
PGM_P title_menu_sensors[] = {title_menu_sensors_GER, title_menu_sensors_EN};
int8_t num_labels_menu_sensors = 5; // MUST match length!
const prog_char labels_menu_sensors_GER1 [] PROGMEM = "PH Referenzpunkt 1";
const prog_char labels_menu_sensors_GER2 [] PROGMEM = "PH Referenzpunkt 2";
const prog_char labels_menu_sensors_GER3 [] PROGMEM = "Min. Durchfluss";
const prog_char labels_menu_sensors_GER4 [] PROGMEM = "D.fluss momentan";
PGM_P PROGMEM labels_menu_sensors_GER [] = {labels_menu_sensors_GER1, labels_menu_sensors_GER2, labels_menu_sensors_GER3, labels_menu_sensors_GER4, labels_menu_back_GER};
const prog_char labels_menu_sensors_EN1 [] PROGMEM = "PH reference 1";
const prog_char labels_menu_sensors_EN2 [] PROGMEM = "PH reference 2";
const prog_char labels_menu_sensors_EN3 [] PROGMEM = "Flowmeter min.";
const prog_char labels_menu_sensors_EN4 [] PROGMEM = "Current flow";
PGM_P PROGMEM labels_menu_sensors_EN [] = {labels_menu_sensors_EN1, labels_menu_sensors_EN2, labels_menu_sensors_EN3, labels_menu_sensors_EN4, labels_menu_back_EN};
PGM_P* labels_menu_sensors[] = {labels_menu_sensors_GER, labels_menu_sensors_EN};

const prog_char title_menu_display [] PROGMEM	= "Display";
int8_t num_labels_menu_display = 4; // MUST match length!
const prog_char labels_menu_display_GER1 [] PROGMEM = "Helligkeit";
const prog_char labels_menu_display_GER2 [] PROGMEM = "Auto. Dimmen";
const prog_char labels_menu_display_GER3 [] PROGMEM = "Hauptschirm anpassen";
PGM_P PROGMEM labels_menu_display_GER [] = {labels_menu_display_GER1, labels_menu_display_GER2, labels_menu_display_GER3, labels_menu_back_GER};
const prog_char labels_menu_display_EN1 [] PROGMEM = "Brightness";
const prog_char labels_menu_display_EN2 [] PROGMEM = "Auto dimming";
const prog_char labels_menu_display_EN3 [] PROGMEM = "Customize mainscreen";
PGM_P PROGMEM labels_menu_display_EN [] = {labels_menu_display_EN1, labels_menu_display_EN2, labels_menu_display_EN3, labels_menu_back_EN};
PGM_P* labels_menu_display[] = {labels_menu_display_GER, labels_menu_display_EN};

// List of available sensor data
const prog_char title_menu_data_GER [] PROGMEM	= "Sensordaten";
const prog_char title_menu_data_EN [] PROGMEM	= "Sensor data";
PGM_P title_menu_data[] = {title_menu_data_GER, title_menu_data_EN};
#if (HW_REVISION < 2)
int8_t num_labels_menu_data = 11; // MUST match length!
const prog_char labels_menu_data_GER1 [] PROGMEM = "Max. Leistung (wei)";
const prog_char labels_menu_data_GER2 [] PROGMEM = "Leistung (wei)";
const prog_char labels_menu_data_GER3 [] PROGMEM = "Max. Leistung (rot)";
const prog_char labels_menu_data_GER4 [] PROGMEM = "Leistung (rot)";
const prog_char labels_menu_data_GER5 [] PROGMEM = "PH Elektrode";
const prog_char labels_menu_data_GER6 [] PROGMEM = "Wassertemperatur";
const prog_char labels_menu_data_GER7 [] PROGMEM = "Systemtemperatur";
const prog_char labels_menu_data_GER8 [] PROGMEM = "Versorgungsspannung";
const prog_char labels_menu_data_GER9 [] PROGMEM = "3,3V-Istwert";
const prog_char labels_menu_data_GER10 [] PROGMEM = "Durchflussmesser";
PGM_P PROGMEM labels_menu_data_GER [] = {labels_menu_data_GER1, labels_menu_data_GER2, labels_menu_data_GER3, 
	labels_menu_data_GER4, labels_menu_data_GER5, labels_menu_data_GER6, labels_menu_data_GER7,
	labels_menu_data_GER8, labels_menu_data_GER9, labels_menu_data_GER10, labels_menu_back_GER};
const prog_char labels_menu_data_EN1 [] PROGMEM = "Max. power (white)";
const prog_char labels_menu_data_EN2 [] PROGMEM = "Cur. power (white)";
const prog_char labels_menu_data_EN3 [] PROGMEM = "Max. power (red)";
const prog_char labels_menu_data_EN4 [] PROGMEM = "Cur. power (red)";
const prog_char labels_menu_data_EN5 [] PROGMEM = "PH Electrode";
const prog_char labels_menu_data_EN6 [] PROGMEM = "Water temperature";
const prog_char labels_menu_data_EN7 [] PROGMEM = "System temperature";
const prog_char labels_menu_data_EN8 [] PROGMEM = "Supply voltage";
const prog_char labels_menu_data_EN9 [] PROGMEM = "3.3V reading";
const prog_char labels_menu_data_EN10 [] PROGMEM = "Flowmeter";
PGM_P PROGMEM labels_menu_data_EN [] = {labels_menu_data_EN1, labels_menu_data_EN2, labels_menu_data_EN3,
	labels_menu_data_EN4, labels_menu_data_EN5, labels_menu_data_EN6, labels_menu_data_EN7,
	labels_menu_data_EN8, labels_menu_data_EN9, labels_menu_data_EN10, labels_menu_back_EN};
#else
int8_t num_labels_menu_data = 16; // MUST match length!
const prog_char labels_menu_data_GER1 [] PROGMEM = "Max. Leistung (wei)";
const prog_char labels_menu_data_GER2 [] PROGMEM = "Leistung (wei)";
const prog_char labels_menu_data_GER3 [] PROGMEM = "Max. Leistung (rot)";
const prog_char labels_menu_data_GER4 [] PROGMEM = "Leistung (rot)";
const prog_char labels_menu_data_GER5 [] PROGMEM = "PH Elektrode";
const prog_char labels_menu_data_GER6 [] PROGMEM = "Wassertemperatur";
const prog_char labels_menu_data_GER7 [] PROGMEM = "Systemtemperatur";
const prog_char labels_menu_data_GER8 [] PROGMEM = "Versorgungsspannung";
const prog_char labels_menu_data_GER9 [] PROGMEM = "Max. Leistung (grn)";
const prog_char labels_menu_data_GER10 [] PROGMEM = "Leistung (grn)";
const prog_char labels_menu_data_GER11 [] PROGMEM = "Max. Leistung (blau)";
const prog_char labels_menu_data_GER12 [] PROGMEM = "Leistung (blau)";
const prog_char labels_menu_data_GER13 [] PROGMEM = "Max. Leistung (total)";
const prog_char labels_menu_data_GER14 [] PROGMEM = "Leistung (total)";
const prog_char labels_menu_data_GER15 [] PROGMEM = "Durchflussmesser";
PGM_P PROGMEM labels_menu_data_GER [] = {labels_menu_data_GER1, labels_menu_data_GER2, labels_menu_data_GER3, 
	labels_menu_data_GER4, labels_menu_data_GER5, labels_menu_data_GER6, labels_menu_data_GER7,
	labels_menu_data_GER8, labels_menu_data_GER9, labels_menu_data_GER10, labels_menu_data_GER11,
	labels_menu_data_GER12, labels_menu_data_GER13, labels_menu_data_GER14, labels_menu_data_GER15, labels_menu_back_GER};
const prog_char labels_menu_data_EN1 [] PROGMEM = "Max. power (white)";
const prog_char labels_menu_data_EN2 [] PROGMEM = "Cur. power (white)";
const prog_char labels_menu_data_EN3 [] PROGMEM = "Max. power (red)";
const prog_char labels_menu_data_EN4 [] PROGMEM = "Cur. power (red)";
const prog_char labels_menu_data_EN5 [] PROGMEM = "PH Electrode";
const prog_char labels_menu_data_EN6 [] PROGMEM = "Water temperature";
const prog_char labels_menu_data_EN7 [] PROGMEM = "System temperature";
const prog_char labels_menu_data_EN8 [] PROGMEM = "Supply voltage";
const prog_char labels_menu_data_EN9 [] PROGMEM = "Max. power (green)";
const prog_char labels_menu_data_EN10 [] PROGMEM = "Cur. power (green)";
const prog_char labels_menu_data_EN11 [] PROGMEM = "Max. power (blue)";
const prog_char labels_menu_data_EN12 [] PROGMEM = "Cur. power (blue)";
const prog_char labels_menu_data_EN13 [] PROGMEM = "Max. power (total)";
const prog_char labels_menu_data_EN14 [] PROGMEM = "Cur. power (total)";
const prog_char labels_menu_data_EN15 [] PROGMEM = "Flowmeter";
PGM_P PROGMEM labels_menu_data_EN [] = {labels_menu_data_EN1, labels_menu_data_EN2, labels_menu_data_EN3,
	labels_menu_data_EN4, labels_menu_data_EN5, labels_menu_data_EN6, labels_menu_data_EN7,
	labels_menu_data_EN8, labels_menu_data_EN9, labels_menu_data_EN10, labels_menu_data_EN11,
	labels_menu_data_EN12, labels_menu_data_EN13, labels_menu_data_EN14, labels_menu_data_EN15, labels_menu_back_EN};
#endif
PGM_P* labels_menu_data[] = {labels_menu_data_GER, labels_menu_data_EN};

const prog_char title_menu_system [] PROGMEM	= "System";
int8_t num_labels_menu_system = 4; // MUST match length!
const prog_char labels_menu_system_GER1 [] PROGMEM = "Systeminformationen";
const prog_char labels_menu_system_2 [] PROGMEM = "Auto FW upgrade";
const prog_char labels_menu_system_3 [] PROGMEM = "SOFTWARE RESET";
PGM_P PROGMEM labels_menu_system_GER [] = {labels_menu_system_GER1, labels_menu_system_2, labels_menu_system_3, labels_menu_back_GER};
const prog_char labels_menu_system_EN1 [] PROGMEM = "System informations";
PGM_P PROGMEM labels_menu_system_EN [] = {labels_menu_system_EN1, labels_menu_system_2, labels_menu_system_3, labels_menu_back_EN};
PGM_P* labels_menu_system[] = {labels_menu_system_GER, labels_menu_system_EN};

const prog_char title_menu_brightness_GER [] PROGMEM = "Helligkeitswert einstellen";
const prog_char title_menu_brightness_EN [] PROGMEM	= "Adjust brightness level";
PGM_P title_menu_brightness[] = {title_menu_brightness_GER, title_menu_brightness_EN};

const prog_char title_menu_auto_off_GER [] PROGMEM = "Auto. Abschaltung setzen";
const prog_char title_menu_auto_off_EN [] PROGMEM	= "Set auto off";
PGM_P title_menu_auto_off[] = {title_menu_auto_off_GER, title_menu_auto_off_EN};

const prog_char unit_auto_off_GER [] PROGMEM = " Sekunden";
const prog_char unit_auto_off_EN [] PROGMEM	= " Seconds";
PGM_P unit_auto_off[] = {unit_auto_off_GER, unit_auto_off_EN};

const prog_char info_GER [] PROGMEM = "Hinweis";
const prog_char info_EN  [] PROGMEM = "Information";
PGM_P info[] = {info_GER, info_EN};

const prog_char error_GER [] PROGMEM = "Fehlermeldung";
const prog_char error_EN  [] PROGMEM = "Error message";
PGM_P error[] = {error_GER, error_EN};

const prog_char browser_title_GER [] PROGMEM = "Dateibrowser";
const prog_char browser_title_EN  [] PROGMEM = "File browser";
const prog_char browser_wait_init_GER [] PROGMEM = "ffne SD-Karte...";
const prog_char browser_wait_init_EN [] PROGMEM = "Opening SD-Card...";
const prog_char browser_error_init_GER [] PROGMEM = "Initialisierungsfehler";
const prog_char browser_error_init_EN [] PROGMEM  = "Could not init. Card";
const prog_char browser_error_fat_GER [] PROGMEM =  "Dateisystem unbekannt";
const prog_char browser_error_fat_EN [] PROGMEM  =  "Unknown file system";
const prog_char browser_error_nofiles_GER [] PROGMEM =  "Keine Dateien gefunden";
const prog_char browser_error_nofiles_EN [] PROGMEM  =  "No files found";
PGM_P browser_title[] = {browser_title_GER, browser_title_EN};
PGM_P browser_wait_init[] = {browser_title_GER, browser_title_EN};
PGM_P browser_error_init[] = {browser_error_init_GER, browser_error_init_EN};
PGM_P browser_error_fat[] = {browser_error_fat_GER, browser_error_fat_EN};
PGM_P browser_error_nofiles[] = {browser_error_nofiles_GER, browser_error_nofiles_EN};

const prog_char title_menu_ph_GER [] PROGMEM	= "PH Referenz";
const prog_char title_menu_ph_EN  [] PROGMEM	= "PH Reference";
PGM_P title_menu_ph[] = {title_menu_ph_GER, title_menu_ph_EN};

const prog_char title_menu_flow_GER [] PROGMEM	= "Setze aktuelle Flussrate";
const prog_char title_menu_flow_EN  [] PROGMEM	= "Set current flow rate";
PGM_P title_menu_flow[] = {title_menu_flow_GER, title_menu_flow_EN};

const prog_char title_menu_min_flow_GER [] PROGMEM	= "Setze Untergrenze";
const prog_char title_menu_min_flow_EN  [] PROGMEM	= "Set minimum flow rate";
PGM_P title_menu_min_flow[] = {title_menu_min_flow_GER, title_menu_min_flow_EN};

const prog_char file_error_msg_EN [] PROGMEM  = "Could not access file";
const prog_char file_error_msg_GER [] PROGMEM = "Dateilesefehler";
PGM_P file_error_msg [] = {file_error_msg_GER, file_error_msg_EN};

const prog_char title_import_EN [] PROGMEM = "Importing profile";
const prog_char title_import_GER [] PROGMEM = "Importiere Profil";
PGM_P title_import [] = {title_import_GER, title_import_EN};

/*const prog_char file_read_msg_EN [] PROGMEM  = "Done!";
const prog_char file_read_msg_GER [] PROGMEM = "Fertig!";
PGM_P file_read_msg [] = {file_read_msg_GER, file_read_msg_EN};*/

const prog_char file_size_msg_EN [] PROGMEM  = "Wrong file size:";
const prog_char file_size_msg_GER [] PROGMEM = "Falsche Dateigre:";
PGM_P file_size_msg [] = {file_size_msg_GER, file_size_msg_EN};

//Global variable needed for the file browser
uint16_t file_index = 0;
uint16_t num_of_files = 0;
int16_t offset = 0;
char file_names[3][13];

//Change default language
void set_language(uint8_t language){
	eeprom_write_byte((uint8_t*)EE_I18N, language);
	current_language = language;
}

uint8_t get_language(void){
	return current_language;
}

/**
 * Loads a string from PROGMEM to SRAM
 */
char *load_string(PGM_P pstring){
	static char string[24];
	unsigned char t;
	uint8_t i=0;
	while((t = pgm_read_byte(pstring++)) && i<23){
		string[i++] = t;
	}
	string[i] = '\0';
	return string;
}

uint8_t file_browser_init(void){
	uint8_t rval;
	spi_set_highspeed();
	for(uint8_t i = 0; i<7;i++){
		wdt_disable(); //We might need more time here
		render_messagebox(info[current_language],browser_wait_init[current_language]);
		if(!mmc_init()){
			render_messagebox(error[current_language], browser_error_init[current_language]);
			rval = FALSE;
		}else{
			rval = TRUE;
		}
		
		wdt_enable(WDTO_2S);
		wdt_reset();
		
		//We got two seconds for FAT loading
		if(rval && !fat_loadFatData()){
			render_messagebox(error[current_language], browser_error_fat[current_language]);
			rval = FALSE;
		}else{
			rval = TRUE;
		}
		
		button = BUTTON_NONE;
		offset = 0;
		
		if(!rval){
			cur_menu = MENU_PROFILES_LEAF;
			spi_slow_down(); //Retry with slower speed
		}else{
			wdt_reset(); //and two for counting
			num_of_files = fat_countFilesInDir();
			if(num_of_files == 0){
				render_messagebox(error[current_language], browser_error_nofiles[current_language]);
				cur_menu = MENU_PROFILES_LEAF;
				rval = FALSE;
			}
			file_index = 0;
			render_redraw = TRUE;
			break;
		}
	}
	return rval;
}

/**
 * Chooses a file and loads it if one is selected
 */
void file_browser(void){
	char converted_file_names[3][22];
	
	unsigned short i,j,k;
	
	if(render_redraw){
		wdt_enable(WDTO_2S);
		wdt_reset();
		fat_loadThreeFileNamesFromDir(file_names, offset);
		//Converting file names
		for(i=0;i<3;i++){
			k=0;
			for(j=0;j<13;j++){
				if(file_names[i][j]=='\0'){
					break;
				}else if(j < 12 && file_names[i][j]==' ' && file_names[i][j+1]>='A' && file_names[i][j+1]<='Z'){
					converted_file_names[i][k++] = '.';
				}else if(file_names[i][j]!=' '){
					converted_file_names[i][k++] = file_names[i][j];
				}
			}
			converted_file_names[i][k] = '\0'; //Terminate string
		}
		render_windowed_browser(load_string(browser_title[current_language]), converted_file_names, offset, file_index, num_of_files);
		//Set Watchdog to 1s timeout reset
		wdt_enable(WDTO_2S);
	}
	
	render_redraw = FALSE;
	
	switch(button){
		case BUTTON_DOWN:
			pwm_reset_idle_time();
			render_redraw = TRUE;
			if(file_index < num_of_files-1){
				file_index++;
				while(offset+2 < file_index){
					offset++;
				}
			}
			break;
		
		case BUTTON_UP:
			pwm_reset_idle_time();
			render_redraw = TRUE;
			if(file_index > 0){
				file_index--;
				while(offset > file_index){
					offset--;
				}
			}
			break;
		
		case BUTTON_RIGHT:
		case BUTTON_CENTER:
			pwm_reset_idle_time();
			//wdt_disable(); //We might need more time here
			render_redraw = TRUE;
			//Check if it is an directory
			///TODO fix cd-command
		/*	if(fat_cd((unsigned char*)file_names[file_index])){
				offset = file_index = 0;
				num_of_files = fat_countFilesInDir();
			}else{*/
				//Loading file
				cur_menu = MENU_PROFILES_TARGET;
				active_index = 0;
				render_menuscreen();
		//	}
			
			//Set Watchdog to 1s timeout reset
			
			break;
			
		case BUTTON_LEFT:
			pwm_reset_idle_time();
			cur_menu = MENU_PROFILES;
			render_menuscreen();
			break;
	}
}

void brightness_editor(uint8_t button_input){

	render_editor_brightness(title_menu_brightness[current_language],pwm_backlight_get());
	switch(button_input){
		case BUTTON_DOWN:
			pwm_reset_idle_time();
			pwm_backlight_set((pwm_backlight_get()-1));
			//Store new settings
			pwm_backlight_set_default(pwm_backlight_get());
			render_editor_brightness(title_menu_brightness[current_language],pwm_backlight_get());
			break;
		case BUTTON_UP:
			pwm_reset_idle_time();
			pwm_backlight_set((pwm_backlight_get()+1));
			//Store new settings
			pwm_backlight_set_default(pwm_backlight_get());
			render_editor_brightness(title_menu_brightness[current_language], pwm_backlight_get());
			break;

		case BUTTON_LEFT:
			pwm_reset_idle_time();
		case BUTTON_CENTER:
			pwm_reset_idle_time();
			cur_menu = MENU_DISPLAY;
			active_index = 0;
			render_menuscreen();
		break;
	}
}

void auto_off_editor(uint8_t button_input){

	render_editor_auto_off(title_menu_auto_off[current_language],unit_auto_off[current_language],pwm_get_auto_off_idle_timeout());
	switch(button_input){
		case BUTTON_DOWN:
			pwm_reset_idle_time();
			pwm_set_auto_off_idle_timeout((pwm_get_auto_off_idle_timeout()+1));
			render_editor_auto_off(title_menu_auto_off[current_language],unit_auto_off[current_language],pwm_get_auto_off_idle_timeout());
			break;
		case BUTTON_UP:
			pwm_reset_idle_time();
			pwm_set_auto_off_idle_timeout((pwm_get_auto_off_idle_timeout()+1));
			render_editor_auto_off(title_menu_auto_off[current_language],unit_auto_off[current_language],pwm_get_auto_off_idle_timeout());
			break;

		case BUTTON_LEFT:
			pwm_reset_idle_time();
		case BUTTON_CENTER:
			pwm_reset_idle_time();
			cur_menu = MENU_DISPLAY;
			render_menuscreen();
		break;
	}

}

void ph_editor(uint8_t button_input){
	static float ph_ref;
	static int16_t ph_value;
	static char editor_init = FALSE;
	pwm_reset_idle_time();
	//Init values on first run
	if(!editor_init){
		editor_init = TRUE;
		if(cur_menu == MENU_PH1){
			ph_ref = ((float)eeprom_read_word((uint16_t*)EE_PH_REF1)) / 100.0f;
			ph_value = eeprom_read_word((uint16_t*)EE_PH_VAL1);
		}else{
			ph_ref = ((float)eeprom_read_word((uint16_t*)EE_PH_REF2)) / 100.0f;
			ph_value = eeprom_read_word((uint16_t*)EE_PH_VAL2);
		}
		
		if(ph_value > 1023 || ph_value < 0
		|| ph_ref > 14.0f || ph_ref < 0.0f){
			//Defaulting invalid presets
			ph_value = 512;
			ph_ref = 7.0f;
		}
	}
	
	switch(active_index){
		case 0:
			switch(button_input){
				case BUTTON_DOWN:
					ph_ref -= 0.1f;
					break;
					
				case BUTTON_UP:
					ph_ref += 0.1f;
					break;
				
				case BUTTON_LEFT:
					if(cur_menu == MENU_PH1){
						active_index = 0;
						eeprom_busy_wait();
						eeprom_write_word((uint16_t*)EE_PH_REF1, (uint16_t)(ph_ref*100.0f));
						eeprom_busy_wait();
						eeprom_write_word((uint16_t*)EE_PH_VAL1, (uint16_t)ph_value);
						eeprom_busy_wait();
					}else{
						active_index = 1;
						eeprom_busy_wait();
						eeprom_write_word((uint16_t*)EE_PH_REF2, (uint16_t)(ph_ref*100.0f));
						eeprom_busy_wait();
						eeprom_write_word((uint16_t*)EE_PH_VAL2, (uint16_t)ph_value);
						eeprom_busy_wait();
					}
					initPH(); //Load new settings
					editor_init = FALSE; //REINIT NEXT TIME!
					cur_menu = MENU_SENSORS;
					render_menuscreen();
					break;
				
				case BUTTON_RIGHT:
					//Goto value
					active_index = 1;
					break;
			}
			break;
			
		case 1:
			switch(button_input){
				case BUTTON_DOWN:
					ph_value--;
					break;
					
				case BUTTON_UP:
					ph_value++;
					break;
				
				case BUTTON_LEFT:
					//Goto PH
					active_index = 0;
					break;
				
				case BUTTON_CENTER:
					//Read value
					ph_value = (uint16_t) ext_raw_ph;
					break;
				
				case BUTTON_RIGHT:
					//Restore value
					if(cur_menu == MENU_PH1){
						ph_value = eeprom_read_word((uint16_t*)EE_PH_VAL1);
					}else{
						ph_value = eeprom_read_word((uint16_t*)EE_PH_VAL2);
					}
					break;
			}
			break;
	}
	//Render if cur_menu hasn't changed
	if(cur_menu == MENU_PH1 || cur_menu == MENU_PH2){
		render_editor_ph(title_menu_ph[current_language], active_index, ph_ref, ph_value);
	}
}

void flowmeter_editor(uint8_t button_input){
	static uint16_t current_rate= 1000;
	pwm_reset_idle_time();
	//Init values on first run
	
	switch(active_index){
		case 1:
			switch(button_input){
				case BUTTON_DOWN:
					if(current_rate > 10)
						current_rate -= 10;
					else
						current_rate = 0;
					break;
				
				case BUTTON_UP:
					current_rate += 10;
					break;
				
				case BUTTON_LEFT:
					//Goto PH
					active_index = 0;
					break;
								
				case BUTTON_RIGHT:
					//Restore value
					active_index = 2;
					break;
			}
			break;
			
		case 2:
			switch(button_input){
				case BUTTON_DOWN:
					if(current_rate > 1)
						current_rate -= 1;
					break;
				
				case BUTTON_UP:
					current_rate += 1;
					break;
				
				case BUTTON_LEFT:
					//Goto PH
					active_index = 1;
					break;
			}
			break;
			
		default:
			switch(button_input){
				case BUTTON_DOWN:
					if(current_rate > 100)
						current_rate -= 100;
					else
						current_rate = 0;
					break;
					
				case BUTTON_UP:
					current_rate += 100;
					break;
				
				case BUTTON_LEFT:
					active_index = 3;
					flowmeter_liter_per_tick = (float)current_rate / FLOWMETER_UPSCALE / (float)flowmeter_last_ticks;
					eeprom_busy_wait();
					eeprom_write_word((uint16_t*)EE_FLOWMETER_LPK, (uint16_t)(flowmeter_liter_per_tick * 10000.0));
					eeprom_busy_wait();
					cur_menu = MENU_SENSORS;
					render_menuscreen();
					break;
				
				case BUTTON_RIGHT:
					//Goto value
					active_index = 1;
					break;
				
				default:
					active_index = 0;
			}
			break;
	}
	//Render if cur_menu hasn't changed
	if(cur_menu == MENU_FLOWMETER_SET){
		render_editor_flow(title_menu_flow[current_language], active_index, current_rate);
	}
}

void min_flow_editor(uint8_t button_input){
	pwm_reset_idle_time();
	//Init values on first run
	
	switch(active_index){
		case 1:
			switch(button_input){
				case BUTTON_DOWN:
					if(flowmeter_rate_min > 10)
						flowmeter_rate_min -= 10;
					else
						flowmeter_rate_min = 0;
					break;
				
				case BUTTON_UP:
					flowmeter_rate_min += 10;
					break;
				
				case BUTTON_LEFT:
					//Goto PH
					active_index = 0;
					break;
								
				case BUTTON_RIGHT:
					//Restore value
					active_index = 2;
					break;
			}
			break;
			
		case 2:
			switch(button_input){
				case BUTTON_DOWN:
					if(flowmeter_rate_min > 1)
						flowmeter_rate_min -= 1;
					break;
				
				case BUTTON_UP:
					flowmeter_rate_min += 1;
					break;
				
				case BUTTON_LEFT:
					//Goto PH
					active_index = 1;
					break;
			}
			break;
			
		default:
			switch(button_input){
				case BUTTON_DOWN:
					if(flowmeter_rate_min > 100)
						flowmeter_rate_min -= 100;
					else
						flowmeter_rate_min = 0;
					break;
					
				case BUTTON_UP:
					flowmeter_rate_min += 100;
					break;
				
				case BUTTON_LEFT:
					active_index = 2;
					eeprom_write_word((uint16_t*)EE_FLOWMETER_MIN, (uint16_t)(flowmeter_rate_min));
					eeprom_busy_wait();
					cur_menu = MENU_SENSORS;
					render_menuscreen();
					break;
				
				case BUTTON_RIGHT:
					//Goto value
					active_index = 1;
					break;
				
				default:
					active_index = 0;
			}
			break;
	}
	//Render if cur_menu hasn't changed
	if(cur_menu == MENU_FLOWMETER_MIN){
		render_editor_flow(title_menu_min_flow[current_language], active_index, flowmeter_rate_min);
	}
}


/**
 * Manually alter channels
 */
void channel_editor(void){
	volatile uint16_t *channel = &BRIGHTNESS_WHITE;
	mode = MODE_NO_CHANNEL_UPDATE; //Change mode to avoid automatic updates following the profile
	
	switch(active_index){
		case 1:
			channel = &BRIGHTNESS_RED;
		break;
		
		case 2:
			channel = &BRIGHTNESS_GREEN;
		break;
		
		case 3:
			channel = &BRIGHTNESS_BLUE;
		break;
	}
	
	switch(button){
		case BUTTON_UP:
			if(active_index > 0){
				active_index--;
			}
		break;
		
		case BUTTON_DOWN:
			if(active_index < 3){
				active_index++;
			}
		break;
		
		case BUTTON_RIGHT:
			if(*channel < 64880){
				*channel += 655;
			}else{
				*channel = 65535;
			}
		break;
		
		case BUTTON_LEFT:
			if(*channel > 655){
				*channel -= 655;
			}else{
				*channel = 0;
			}
		break;
		
		case BUTTON_CENTER: //Exit
			active_index = 4;
			cur_menu = MENU_MAIN;
			mode = MODE_STANDARD;
			render_menuscreen();
			return;
		break;
	}
	
	render_testscreen(active_index);
}

/**
 * END OF REALTIME TRIGGERED SUBAPPS
 */

void clock_editor(uint8_t button_input){
	date current_rtc = ds1302_get_date();
	render_editor_date(title_menu_clock[current_language], active_index, current_rtc);
	switch(button_input){
		case BUTTON_DOWN:
			pwm_reset_idle_time();
			switch(active_index){
				case 0:
					current_rtc.hrs--;
					break;
				
				case 1:
					current_rtc.min--;
					break;
					
				case 2:
					current_rtc.sec--;
					break;
				
				case 3:
					current_rtc.day--;
					break;
					
				case 4:
					current_rtc.month--;
					break;
					
				case 5:
					current_rtc.year--;
					break;
			}
			ds1302_set_date(current_rtc);
			break;
			
		case BUTTON_UP:
			pwm_reset_idle_time();
			switch(active_index){
				case 0:
					current_rtc.hrs++;
					break;
				
				case 1:
					current_rtc.min++;
					break;
					
				case 2:
					current_rtc.sec++;
					break;
				
				case 3:
					current_rtc.day++;
					break;
					
				case 4:
					current_rtc.month++;
					break;
					
				case 5:
					current_rtc.year++;
					break;
			}
			ds1302_set_date(current_rtc);
			break;
			
			case BUTTON_LEFT:
				pwm_reset_idle_time();
				if(active_index > 0){
					active_index--;
				}else{
					cur_menu = MENU_CLOCK;
					active_index = 0;
					render_menuscreen();
				}
				break;
			
			case BUTTON_RIGHT:
				pwm_reset_idle_time();
				if(active_index < 5){
					active_index++;
				}
	}
}

/**
 * Manages Jumpes between menues.
 */
void goto_next_menu(){
	uint8_t byte = 0;
	uint16_t word = 0;
	int8_t profile_load_result;
	char name[9];
	anim = ANIM_PUSH_LEFT;
	switch(cur_menu){//In what menu are we at this moment ?!
		// Handling of Mainscreen (in no menu yet)
		case MAINSCREEN: // From mainscreen, swich to main menu
			cur_menu = MENU_MAIN;
			active_index = 0;
			break;


		//Handling of main menu
		case MENU_MAIN: // This is the main menu.
			switch(active_index){

				case 0: // jump to profiles menu
					cur_menu = MENU_PROFILES;
					active_index = 0;
					break;
					
				case 1:
					cur_menu = MENU_OPTIONS;
					active_index = 0;
					break;

				case 2: //render credits
					cur_menu = MENU_CREDITS;
				break;
				
				case 3: // Exit menu, back to main screen
					speed_factor = 1000;
					cur_menu = MENU_DEMO;
				break;
				
				case 4: //Go to test mode
					cur_menu = MENU_TEST;
					active_index = 0;
					button = BUTTON_NONE;
				break;
				
				case 5: // Exit menu, back to main screen
					anim = ANIM_PUSH_RIGHT;
					cur_menu = MAINSCREEN;
					active_index = 0;
				break;
			}
			break; // End of main menu handling
			
		case MENU_OPTIONS:
			switch(active_index){
				case 0:
					cur_menu = MENU_LANGUAGE;
					active_index = 0;
					break;
				
				case 1:
					cur_menu = MENU_CLOCK;
					active_index = 0;
					break;
				
				case 2:
					cur_menu = MENU_SENSORS;
					active_index = 0;
					break;
					
				case 3:
					cur_menu = MENU_DISPLAY;
					active_index = 0;
					break;
					
				case 4:
					cur_menu = MENU_SYSTEM;
					active_index = 0;
					break;
					
				default:
					anim = ANIM_PUSH_RIGHT;
					cur_menu = MENU_MAIN;
					active_index = 1;
					break;
			}
			break;
		
		case MENU_MAINSCREEN:
			switch(active_index){
				case 0:
					cur_menu = MENU_DATA_1;
					active_index = 0;
					break;
				
				case 1:
					cur_menu = MENU_DATA_2;
					active_index = 0;
					break;
				
				case 2:
					cur_menu = MENU_DATA_3;
					active_index = 0;
					break;
				
				case 3:
					cur_menu = MENU_DATA_4;
					active_index = 0;
					break;
			}
		break;
		
		case MENU_DATA_1:
			word = EE_MAINSCREEN_SLOT1;
		case MENU_DATA_2:
			if(!word){
				word = EE_MAINSCREEN_SLOT2;
			}
		case MENU_DATA_3:
			if(!word){
				word = EE_MAINSCREEN_SLOT3;
			}
		case MENU_DATA_4:
			if(!word){
				word = EE_MAINSCREEN_SLOT4;
			}
			
			switch(active_index){
				case 0:
					eeprom_write_byte((uint8_t*)word, SENSOR_MAX_POWER_WHITE);
					eeprom_busy_wait();
					break;
				
				case 1:
					eeprom_write_byte((uint8_t*)word, SENSOR_POWER_WHITE);
					eeprom_busy_wait();
					break;
				
				case 2:
					eeprom_write_byte((uint8_t*)word, SENSOR_MAX_POWER_RED);
					eeprom_busy_wait();
					break;
				
				case 3:
					eeprom_write_byte((uint8_t*)word, SENSOR_POWER_RED);
					eeprom_busy_wait();
					break;
				
				case 4:
					eeprom_write_byte((uint8_t*)word, SENSOR_PH);
					eeprom_busy_wait();
					break;
				
				case 5:
					eeprom_write_byte((uint8_t*)word, SENSOR_WATER_TEMPERATURE);
					eeprom_busy_wait();
					break;
				
				case 6:
					eeprom_write_byte((uint8_t*)word, SENSOR_SYSTEM_TEMPERATURE);
					eeprom_busy_wait();
					break;
				
				case 7:
					eeprom_write_byte((uint8_t*)word, SENSOR_VOLTAGE_12V);
					eeprom_busy_wait();
					break;
				
				#if (HW_REVISION < 2)
				case 8:
					eeprom_write_byte((uint8_t*)word, SENSOR_VOLTAGE_3V);
					eeprom_busy_wait();
					break;
					
				case 9:
					eeprom_write_byte((uint8_t*)word, SENSOR_FLOWMETER);
					eeprom_busy_wait();
					break;
				#else
				case 8:
					eeprom_write_byte((uint8_t*)word, SENSOR_MAX_POWER_GREEN);
					eeprom_busy_wait();
					break;
				
				case 9:
					eeprom_write_byte((uint8_t*)word, SENSOR_POWER_GREEN);
					eeprom_busy_wait();
					break;
				
				case 10:
					eeprom_write_byte((uint8_t*)word, SENSOR_MAX_POWER_BLUE);
					eeprom_busy_wait();
					break;
				
				case 11:
					eeprom_write_byte((uint8_t*)word, SENSOR_POWER_BLUE);
					eeprom_busy_wait();
					break;
				
				case 12:
					eeprom_write_byte((uint8_t*)word, SENSOR_MAX_POWER_TOTAL);
					eeprom_busy_wait();
					break;
				
				case 13:
					eeprom_write_byte((uint8_t*)word, SENSOR_POWER_TOTAL);
					eeprom_busy_wait();
					break;
					
				case 14:
					eeprom_write_byte((uint8_t*)word, SENSOR_FLOWMETER);
					eeprom_busy_wait();
					break;
				#endif
				
				default:
					anim = ANIM_PUSH_RIGHT;
					break;
			}
			active_index = cur_menu - MENU_DATA_1;
			cur_menu = MENU_MAINSCREEN;
		break;
		
		case MENU_LANGUAGE:
			if(active_index < num_of_languages){
				set_language(active_index);
			}
			cur_menu = MENU_OPTIONS;
			active_index = 0;
			break;
			
		case MENU_CLOCK:
			switch(active_index){
				case 0:
					active_index = 0;
					cur_menu = MENU_CLOCK_EDITOR;
					break;
				
				case 1:
					//Toggle clock radio receiver
					anim = ANIM_FLIP;
					byte = eeprom_read_byte((uint8_t*)EE_TIME_SETTINGS);
					if(byte & EE_TIME_FLAG_DCF){
						byte &= ~EE_TIME_FLAG_DCF;
					}else{
						byte |= EE_TIME_FLAG_DCF;
					}
					eeprom_write_byte((uint8_t*)EE_TIME_SETTINGS, byte);
					break;
					
				case 2:
					//Toggle auto DST settings
					anim = ANIM_FLIP;
					byte = eeprom_read_byte((uint8_t*)EE_TIME_SETTINGS);
					if(byte&EE_TIME_FLAG_AUTODST){
						byte &= ~EE_TIME_FLAG_AUTODST;
					}else{
						byte |= EE_TIME_FLAG_AUTODST;
					}
					eeprom_write_byte((uint8_t*)EE_TIME_SETTINGS, byte);
					break;
					
				case 3:
					anim = ANIM_FLIP;
					/*Do nothing! This is just to see if DST ist enabled or not.
					  The DST-flag must not be changed manually in auto-dst mode
					  and doesn't make sense in manual mode.*/
					break;
					
				case 4:
					//Toggle moon-phase usage
					anim = ANIM_FLIP;
					byte = eeprom_read_byte((uint8_t*)EE_TIME_SETTINGS);
					if(byte&EE_TIME_USE_MOON_PHASE){
						byte &= ~EE_TIME_USE_MOON_PHASE;
						use_moon_phase = FALSE;
					}else{
						byte |= EE_TIME_USE_MOON_PHASE;
						use_moon_phase = TRUE;
					}
					eeprom_write_byte((uint8_t*)EE_TIME_SETTINGS, byte);
					break;
					
				default:
					anim = ANIM_PUSH_RIGHT;
					active_index = 1;
					cur_menu = MENU_OPTIONS;
					break;
			}
			break;

		//Handling of profiles Menu
		case MENU_PROFILES: // Directly loads LED Profiles into mem and displays the result
			switch(active_index){
				case 0:
					cur_menu = MENU_PROFILES_LEAF;
					profile_load_result = load_profile(PROFILE_AMAZONAS);
					render_profile_loading_result(load_string((char*)pgm_read_word(&(labels_menu_profiles[current_language][active_index]))),profile_load_result);
					break;

				case 1:
					cur_menu = MENU_PROFILES_LEAF;
					profile_load_result = load_profile(PROFILE_PACIFIC);
					render_profile_loading_result(load_string((char*)pgm_read_word(&(labels_menu_profiles[current_language][active_index]))),profile_load_result);
					break;

				case 2:
					cur_menu = MENU_PROFILES_LEAF;
					profile_load_result = load_profile(PROFILE_COLOGNE);
					render_profile_loading_result(load_string((char*)pgm_read_word(&(labels_menu_profiles[current_language][active_index]))),profile_load_result);
					break;
				
				case 3:
					cur_menu = MENU_PROFILES_LEAF;
					profile_load_result = load_profile(PROFILE_CONGO);
					render_profile_loading_result(load_string((char*)pgm_read_word(&(labels_menu_profiles[current_language][active_index]))),profile_load_result);
					break;
				
				case 4:
					cur_menu = MENU_PROFILES_LEAF;
					profile_load_result = load_profile(PROFILE_USER_1);
					if(profile_load_result == LOAD_SUCCESS){
						get_profile_name(PROFILE_USER_1, name);
						render_profile_loading_result(name,profile_load_result);
					}else{
						render_profile_loading_result(load_string((char*)pgm_read_word(&(labels_menu_profiles[current_language][active_index]))),profile_load_result);
					}
					break;

				case 5:
					cur_menu = MENU_PROFILES_LEAF;
					profile_load_result = load_profile(PROFILE_USER_2);
					if(profile_load_result == LOAD_SUCCESS){
						get_profile_name(PROFILE_USER_2, name);
						render_profile_loading_result(name,profile_load_result);
					}else{
						render_profile_loading_result(load_string((char*)pgm_read_word(&(labels_menu_profiles[current_language][active_index]))),profile_load_result);
					}
					break;
				
				case 6:
					cur_menu = MENU_PROFILES_LEAF;
					profile_load_result = load_profile(PROFILE_USER_3);
					if(profile_load_result == LOAD_SUCCESS){
						get_profile_name(PROFILE_USER_3, name);
						render_profile_loading_result(name,profile_load_result);
					}else{
						render_profile_loading_result(load_string((char*)pgm_read_word(&(labels_menu_profiles[current_language][active_index]))),profile_load_result);
					}
					break;

				case 7:
					if(file_browser_init()){ //FB need initialization!
						cur_menu = MENU_FILE_BROWSER;
					}
					break;

				case 8: // Back to Main Menu
					anim = ANIM_PUSH_RIGHT;
					cur_menu = MENU_MAIN;
					active_index = 0;
					break;
			}
			break;//End of profiles menu handling
			
		case MENU_PROFILES_TARGET:
			switch(active_index){
				case 0:
					switch(import_profile(file_names[file_index], PROFILE_USER_1)){
						case IMPORT_SUCCESS:
							profile_load_result = load_profile(PROFILE_USER_1);
							if(profile_load_result == LOAD_SUCCESS){
								get_profile_name(PROFILE_USER_1, name);
								render_profile_loading_result(name,profile_load_result);
							}else{
								render_profile_loading_result(load_string((char*)pgm_read_word(&(labels_menu_profiles[current_language][active_index]))),profile_load_result);
							}
							break;
						
						case IMPORT_NO_FILE:
							render_messagebox(title_import[current_language], file_error_msg[current_language]);
							break;
							
						case IMPORT_INSUFFICIENT_LENGTH:
							render_messagebox(title_import[current_language], file_size_msg[current_language]);
							break;
					}
					//active_index = 7;
					break;

				case 1:
					switch(import_profile(file_names[file_index], PROFILE_USER_2)){
						case IMPORT_SUCCESS:
							profile_load_result = load_profile(PROFILE_USER_2);
							if(profile_load_result == LOAD_SUCCESS){
								get_profile_name(PROFILE_USER_2, name);
								render_profile_loading_result(name,profile_load_result);
							}else{
								render_profile_loading_result(load_string((char*)pgm_read_word(&(labels_menu_profiles[current_language][active_index]))),profile_load_result);
							}
							break;
						
						case IMPORT_NO_FILE:
							render_messagebox(title_import[current_language], file_error_msg[current_language]);
							break;
						
						case IMPORT_INSUFFICIENT_LENGTH:
							render_messagebox(title_import[current_language], file_size_msg[current_language]);
							break;
					}
					//active_index = 7;
					break;
				
				case 2:
					switch(import_profile(file_names[file_index], PROFILE_USER_3)){
						case IMPORT_SUCCESS:
							profile_load_result = load_profile(PROFILE_USER_3);
							if(profile_load_result == LOAD_SUCCESS){
								get_profile_name(PROFILE_USER_3, name);
								render_profile_loading_result(name,profile_load_result);
							}else{
								render_profile_loading_result(load_string((char*)pgm_read_word(&(labels_menu_profiles[current_language][active_index]))),profile_load_result);
							}
							break;
							
						case IMPORT_NO_FILE:
							render_messagebox(title_import[current_language], file_error_msg[current_language]);
							break;
						
						case IMPORT_INSUFFICIENT_LENGTH:
							render_messagebox(title_import[current_language], file_size_msg[current_language]);
							break;
					}
					//active_index = 7;
					break;
			}
			cur_menu = MENU_PROFILES_LEAF;
			break;
		
			case MENU_DISPLAY:
				switch(active_index){
					case 0:
						cur_menu = MENU_BRIGHTNESS;
						button = BUTTON_NONE;
						break;
					
					case 1:
						//Toggle auto dimming setting
						anim = ANIM_FLIP;
						if(eeprom_read_byte((uint8_t*)EE_DISPLAY_DIMMING)){
							byte = FALSE;
						}else{
							byte = TRUE;
						}
						eeprom_write_byte((uint8_t*)EE_DISPLAY_DIMMING, byte);
						break;
					
					case 2:
						cur_menu = MENU_MAINSCREEN;
						active_index = 0;
						break;
						
					case 3:
						anim = ANIM_PUSH_RIGHT;
						active_index = 3;
						cur_menu = MENU_OPTIONS;
						break;
			}
			break;//End of display menu handling
			
			case MENU_SYSTEM:
				switch(active_index){
					case 0:
						cur_menu = MENU_SYSTEM_INFO;
						break;
					case 1:
						//Toggle auto setting
						anim = ANIM_FLIP;
						if(eeprom_read_byte((uint8_t*)EE_AUTO_FW_UPGRADE)){
							byte = FALSE;
						}else{
							byte = TRUE;
						}
						eeprom_write_byte((uint8_t*)EE_AUTO_FW_UPGRADE, byte);
						break;
					case 2:
						//Force reset by watchdog
						_delay_ms(5000);
						break;
					case 3:
						anim = ANIM_PUSH_RIGHT;
						active_index = 4;
						cur_menu = MENU_OPTIONS;
						break;
			}
			break;
			
			case MENU_SENSORS:
				switch(active_index){
					case 0:
						cur_menu = MENU_PH1;
						active_index = 0;
						break;
						
					case 1:
						cur_menu = MENU_PH2;
						active_index = 0;
						break;
					
					case 2:
						cur_menu = MENU_FLOWMETER_MIN;
						active_index = 0;
						break;
						
					case 3:
						cur_menu = MENU_FLOWMETER_SET;
						break;
					
					case 4:
						anim = ANIM_PUSH_RIGHT;
						active_index = 2;
						cur_menu = MENU_OPTIONS;
						break;
				}
			break;
	};
}

/**
 * Handles "go back" action
 */
void goto_previous_menu(){
	anim = ANIM_PUSH_RIGHT;
	switch(cur_menu){//In what menu are we at this moment ?!

		// Handling of Mainscreen (in no menu yet)
		case MAINSCREEN: // Nothing has changed...
			render_state_changed = 0;
			break;

		// Handling of Main Menu
		case MENU_MAIN: // Go back to MAIN SCREEN
			cur_menu = MAINSCREEN;
			active_index = 0;
			render_mainscreen();
			break;

		// Handling of Profiles Menu
		case MENU_PROFILES: // Go back to Main Menu
			cur_menu = MENU_MAIN;
			active_index = 0;
			break;

		//Handling of Profile Leafs
		case MENU_PROFILES_LEAF:
			cur_menu = MENU_PROFILES;
			active_index = 0;
			break;
			
		case MENU_PROFILES_TARGET:
			cur_menu = MENU_PROFILES;
			active_index = 7;
			break;

		// Handling of CREDITS Menu
		case MENU_CREDITS: // Go back to Main Menu
//		case MENU_CREDITS_LEAF: // Go back to Main Menu
			cur_menu = MENU_MAIN;
			active_index = 2;
			break;
			
		case MENU_OPTIONS:
			cur_menu = MENU_MAIN;
			active_index = 1;
			break;
		
		case MENU_LANGUAGE:
			cur_menu = MENU_OPTIONS;
			active_index = 0;
			break;
		
		case MENU_CLOCK:
			cur_menu = MENU_OPTIONS;
			active_index = 1;
			break;
		
		case MENU_SENSORS:
			cur_menu = MENU_OPTIONS;
			active_index = 2;
			break;
			
		case MENU_FLOWMETER_MIN:
			cur_menu = MENU_SENSORS;
			active_index = 2;
			break;
			
		case MENU_DISPLAY:
			cur_menu = MENU_OPTIONS;
			active_index = 3;
			break;

		case MENU_BRIGHTNESS:
			cur_menu = MENU_DISPLAY;
			active_index = 0;
			break;

		/*case MENU_AUTO_OFF:
			cur_menu = MENU_DISPLAY;
			active_index = 1;
			break;*/
			
		case MENU_DEMO:
			cur_menu = MENU_MAIN;
			speed_factor = 1;
			break;
			
		case MENU_SYSTEM:
			cur_menu = MENU_OPTIONS;
			active_index = 4;
			break;
			
		case MENU_SYSTEM_INFO:
			cur_menu = MENU_SYSTEM;
			active_index = 0;
			break;
		
		case MENU_DATA_1:
			cur_menu = MENU_MAINSCREEN;
			active_index = 0;
			break;
		
		case MENU_DATA_2:
			cur_menu = MENU_MAINSCREEN;
			active_index = 1;
			break;
		
		case MENU_DATA_3:
			cur_menu = MENU_MAINSCREEN;
			active_index = 2;
			break;
		
		case MENU_DATA_4:
			cur_menu = MENU_MAINSCREEN;
			active_index = 3;
			break;
		
		default:
			//This should not happen
			cur_menu = MENU_MAIN;
			active_index = 0;
			break;
	}// END of cur_menu switch
}



/**
 * Handling general rendering of menu screens, the only interface function to Render.c/h !
 */
void render_menuscreen(){
	uint8_t booleans[9]; //Note: Needs to be long enogh for the biggest boolean menu!
	uint8_t byte;
	//char title[15];
	switch(cur_menu){//In what menu are we at this moment ?!

		case MENU_MAIN:
			//booleans[0]=booleans[1]=booleans[2]=booleans[4]=255;
			//booleans[3] = (speed_factor != 1);
			render_menu_P(title_menu_main[current_language],labels_menu_main[current_language],active_index,num_labels_menu_main);
			break;

		case MENU_PROFILES:
			for(byte=0; byte < 7; byte++){
				booleans[byte] = (byte == current_profile);
			}
			booleans[7] = booleans[8] = 255;
			render_boolean_menu_P(title_menu_profiles[current_language],labels_menu_profiles[current_language],booleans,active_index,num_labels_menu_profiles);
			break;
			
		case MENU_PROFILES_TARGET:
			for(byte=0; byte < 7; byte++){
				booleans[byte] = (byte+4 == current_profile);
			}
			booleans[7] = booleans[8] = 255;
			render_boolean_menu_P(title_menu_target_profiles[current_language],labels_menu_target_profiles[current_language],booleans,active_index,num_labels_menu_target_profiles);
			break;
			
		case MENU_OPTIONS:
			render_menu_P(title_menu_options[current_language],labels_menu_options[current_language],active_index,num_labels_menu_options);
			break;
			
		case MENU_LANGUAGE:
			render_menu_P(title_menu_languages[current_language],available_languages_P,active_index,num_of_languages);
			break;
			
		case MENU_CLOCK:
			//Set clock and "back" has no boolean
			booleans[0] = booleans[5] = 255;
			byte = eeprom_read_byte((uint8_t*)EE_TIME_SETTINGS);
			booleans[1] = (byte & EE_TIME_FLAG_DCF);
			booleans[2] = (byte & EE_TIME_FLAG_AUTODST);
			booleans[3] = (byte & EE_TIME_FLAG_DST);
			booleans[4] = (byte & EE_TIME_USE_MOON_PHASE);
			render_boolean_menu_P(title_menu_clock[current_language],labels_menu_clock[current_language],booleans,active_index,num_labels_menu_clock);
			break;
			
		case MENU_SENSORS:
			render_menu_P(title_menu_sensors[current_language],labels_menu_sensors[current_language],active_index,num_labels_menu_sensors);
			break;
		
		case MENU_DISPLAY:
			booleans[0] = booleans[2] = booleans[3] = 255;
			if(eeprom_read_byte((uint8_t*)EE_DISPLAY_DIMMING)){
				booleans[1] = TRUE;
			}else{
				booleans[1] = FALSE;
			}
			render_boolean_menu_P(title_menu_display,labels_menu_display[current_language],booleans,active_index,num_labels_menu_display);
			break;
			
		case MENU_SYSTEM:
			//Set FW and "back" has no boolean
			booleans[0] = booleans[2] = booleans[3] = 255;
			byte = eeprom_read_byte((uint8_t*)EE_AUTO_FW_UPGRADE);
			booleans[1] = byte!=255?byte:1;
			render_boolean_menu_P(title_menu_system,labels_menu_system[current_language],booleans,active_index,num_labels_menu_system);
			break;
			
		case MENU_SYSTEM_INFO:
			render_sysinfos();
			break;
		
		case MENU_BRIGHTNESS:
			brightness_editor(button);
			break;

		/*case MENU_AUTO_OFF:
			auto_off_editor(button);
			break;*/


		case MENU_CREDITS:
//		case MENU_CREDITS_LEAF:
			render_credits();
			break;

		/*case MENU_PROFILES_LEAF:
			render_profile_loading_result(load_string((char*)pgm_read_word(&(labels_menu_profiles[current_language][active_index]))),profile_load_result);
			break;*/
			
		case MENU_CLOCK_EDITOR:
			clock_editor(button);
			break;
			
		case MENU_FILE_BROWSER:
			file_browser();
			break;
			
		case MENU_DEMO:
			render_demoscreen();
			break;
			
		case MENU_PH1:
		case MENU_PH2:
			ph_editor(button);
			break;
			
		case MENU_FLOWMETER_SET:
			flowmeter_editor(button);
			break;
			
		case MENU_FLOWMETER_MIN:
			min_flow_editor(button);
			break;
		
		case MENU_TEST:
			channel_editor();
			break;
		
		case MENU_MAINSCREEN:
			render_mainscreen();
			break;
		
		case MENU_DATA_1:
		case MENU_DATA_2:
		case MENU_DATA_3:
		case MENU_DATA_4:
			render_menu_P(title_menu_data[current_language],labels_menu_data[current_language],active_index,num_labels_menu_data);
			break;
	}
	flip_buffer(anim);
}



/**
 * Handles "One entry down" action
 */
void go_down_within_menu(){
	int8_t num_labels = 1;
	switch(cur_menu){
		case MENU_MAIN:
			num_labels = num_labels_menu_main;
			break;
		
		case MENU_OPTIONS:
			num_labels = num_labels_menu_options;
			break;
			
		case MENU_PROFILES:
			num_labels = num_labels_menu_profiles;
			break;
		
		case MENU_PROFILES_TARGET:
			num_labels = num_labels_menu_target_profiles;
			break;
		
		case MENU_LANGUAGE:
			num_labels = num_of_languages;
			break;
		
		case MENU_CLOCK:
			num_labels = num_labels_menu_clock;
			break;
			
		case MENU_SENSORS:
			num_labels = num_labels_menu_sensors;
			break;
		
		case MENU_DISPLAY:
			num_labels = num_labels_menu_display;
			break;
			
		case MENU_SYSTEM:
			num_labels = num_labels_menu_system;
			break;
		
		case MENU_MAINSCREEN:
			num_labels = 4;
			break;
		
		case MENU_DATA_1:
		case MENU_DATA_2:
		case MENU_DATA_3:
		case MENU_DATA_4:
			num_labels = num_labels_menu_data;
			break;
	}

	if(active_index < (num_labels-1)){
		active_index++;
	}

}



/**
 * Handles "One entry up" action
 */
void go_up_within_menu(){
	if(active_index > 0){
		active_index--;
	}
}



/////////////////////////////////////////////////////////////////////////////////////////////////////
// Never touch code below this line (at least you know what to do :-)  )
/////////////////////////////////////////////////////////////////////////////////////////////////////


/**
 * Most important function:
 * Processes possible input from keypad and renders menuscreen
 */
uint8_t processKeyInput(){
	anim = ANIM_FLIP;
	//Check if language is valid
	if(current_language >= num_of_languages){
		current_language = eeprom_read_byte((uint8_t*)EE_I18N);
		//Defaulting if eeprom-value is not valid
		if(current_language >= num_of_languages){
			set_language(I18N_EN);
		}
	}
	
	render_state_changed = TRUE;
	
	button = button_get_key();
	if(cur_menu < 100){
		switch(button){
			case BUTTON_UP :
				pwm_reset_idle_time();
				go_up_within_menu();
				break;
			
			case BUTTON_DOWN :
				pwm_reset_idle_time();
				go_down_within_menu();
				break;
				
			case BUTTON_RIGHT :
				pwm_reset_idle_time();
				goto_next_menu();
				break;
			
			case BUTTON_CENTER :
				pwm_reset_idle_time();
				goto_next_menu();
				break;
				
			case BUTTON_LEFT:
				pwm_reset_idle_time();
				goto_previous_menu();
				break;
				
			case BUTTON_NONE:
				render_state_changed = FALSE;
				break;
		}
	}
	if(cur_menu > 80){
		render_state_changed = TRUE;
	}
	
	// RENDER any of the menuscreens, if necessary
	if(render_state_changed && is_in_menu() == INSIDE_MENU){
		render_menuscreen();
	}

	return is_in_menu();

}


uint8_t is_in_menu(void){
	if(cur_menu == MAINSCREEN){
		return OUTSIDE_MENU;
	}else{
		return INSIDE_MENU;
	}
}



