/***************************************************************************
 *  Prometeus.c
 *  Copyright  2007-2009 Olli S., Oli Z.
 *  oliver.schwaneberg@gmail.com
 *  ozilken@gmail.com
 *
 *  Teilweise basierend auf den Codebeispielen von ATMEL Norway.
 *
 ***************************************************************************

 ============================ CHANGELOG ====================================
	Version 1.0		Der Urknall
	Version 1.1		Hardware-PWM fÃ¼r WeiÃkanal
	Version 1.2		10Bit Hardware PWM fÃ¼r WeiÃkanal
	Version 1.3beta	Intelligenteres Timing zur Verbesserung des Software-PWM
	Version 1.3		Behobene Absturzursache: Raceconditions im Menu
	* BREAK *
	Version 1.0-LMC Neue Version fÃ¼r neue Hardware: den LeuchtmuffController
	Version 1.01-LMC Mini-Bugfix an Powermanagement und DCF-Fehlerkorrektur
	Version 1.02-LMC Kleine Ãnderungen an der Displaybeleuchtungssteuerung
	* STOP HAMMERTIME *
	Version 1.0-RISE
	* Build 131 bugfix - time toggling time on dst switch (dst off)
 ===========================================================================

 ################### TO IMPLEMENT / CURRENT ISSUES #########################
 * Menustruktur						(X)
 * Knopfeingaben					(X)
 * Beeper						(X)
 * Nutzung des RTC-Chip DS1302				(X)
 * Kapselung einer neuen Bibliothek für DCF77	(?)
 * Implementation des Mondphasenkalendars	(X)
 * Sensorische Eingaben					(X)
	# Interne TemperaturÃ¼berwachung			(X)
	# SpannungsÃ¼berwachung				(X)
	# PH-Sonde auslesen und kalibrieren		(X)
	# Temperatursonde auslesen			(X)
 * Kommunikationsschnittstelle zu Erweiterungsport	( )
 * DOGM Display						(X)
	Grafische Darstellung des MenÃ¼s	(X)
	MenÃ¼Ã¼bergÃ¤nge					(X)
	Startbildschirm					(X)
	Statusangaben und Hinweismeldungen		(X)
 * SD-Kartenzugriff					(X)
 * Umgestaltung der Datenbankstruktur*		(X)
 	# Definition von Kurvenpresets			( )
	# Abstraktion des Datenzugriffs			(X)
		+ Lesen/Schreiben von/auf EEPROM	(X)
		+ Lesen/Schreiben von/auf SD-Karte	(X)
		+ Lesen von PROGMEM / int. Flash	(X)
 ###########################################################################
 
 Ideen:
 * MenÃ¼Ã¼bergÃ¤nge sollten durch die Hintergrundbeleuchtung unterstÃ¼tzt
   werden: Beim Wechseln wird es schnell dunkel und wenn es wieder hell
   wird ist der Bildschirminhalt ausgetauscht.
 */

#include "Prometheus.h"

/**
 * Aktuelle Zeit (Pulse seit Mitternacht)
 * Bemerkung: es ist einfacher die Pulse seit Mitternacht zu
 * verwenden als Millisekunden.
 */
//Pulse pro Tag sind 24h*60m*60s*60p
const int32_t pulses_per_day = 5184000;
int32_t pulses_since_midnight = 0;

date rtc;

//External temperature in celsius
int16_t ext_midnight_temperature = 0;

//External ph probe
float ext_midnight_ph = -1.0f;

//Measure flag to be set up every second
uint8_t sec_flag = FALSE, pulse_flag = FALSE;

//This flag can be used to disable all leds
uint8_t operation = TRUE, mode = MODE_STANDARD;
uint16_t  sys_status_vec = 0;

//This simple counter is helping to provide a second based timing
uint8_t sec_part = 0;

//Flowmeter variables
uint8_t flowmeter_level = 0;
uint16_t flowmeter_ticks = 0, flowmeter_last_ticks;
float flowmeter_rate = 0.0, flowmeter_rate_min = 0.0, flowmeter_liter_per_tick = 0.0;


/**
 * Dieser Faktor kann zum Manipulieren der Ãbergangsgeschwindigkeit
 * genutzt werden. ErmÃ¶glicht den Demo-Modus.
 */
uint16_t speed_factor = 1, moon_phase_factor;

//Store the moon_phase
uint8_t moon_phase, use_moon_phase;

// Am Start sid wir ausserhalb des des menus
uint8_t menu_state = OUTSIDE_MENU;

/* Indizes der aktuellen Werte */
uint8_t profile_index   = -1;

int32_t last_entry_ptime = 0;
int32_t next_entry_ptime = -1;

float  last_entry_red_brightness = 0;
float  next_entry_red_brightness = 0;

float  last_entry_green_brightness = 0;
float  next_entry_green_brightness = 0;

float  last_entry_blue_brightness = 0;
float  next_entry_blue_brightness = 0;

float  last_entry_white_brightness = 0;
float  next_entry_white_brightness = 0;

/**
 * Calculates a factor for the moon-light channel
 */
void calc_moon_phase_factor(void){
	if(moon_phase == 0){
		moon_phase_factor = 12;
	}else if(moon_phase == 1 || moon_phase == 7){
		moon_phase_factor = 8;
	}else if(moon_phase == 2 || moon_phase == 6){
		moon_phase_factor = 4;
	}else if(moon_phase == 3 || moon_phase == 5){
		moon_phase_factor = 2;
	}else{
		moon_phase_factor = 1;
	}
}

/**
 * Reset brightness at the dawn of a new day
 */
void reset_brightness(void){
	//Handle moon phase influences
	moon_phase = calculate_moon_phase(rtc.year, rtc.month, rtc.day);
	calc_moon_phase_factor();
	
	profile_index   = -1;

	last_entry_ptime = 0;
	next_entry_ptime = 0;

	last_entry_red_brightness = BRIGHTNESS_RED;
	next_entry_red_brightness = BRIGHTNESS_RED;

	last_entry_green_brightness = BRIGHTNESS_GREEN;
	next_entry_green_brightness = BRIGHTNESS_GREEN;

	last_entry_blue_brightness = BRIGHTNESS_BLUE;
	next_entry_blue_brightness = BRIGHTNESS_BLUE;

	last_entry_white_brightness = BRIGHTNESS_WHITE;
	next_entry_white_brightness = BRIGHTNESS_WHITE;
	
	//Parasitary Code insertion: Remember current external readings (if valid)
	if(ext_ph > 4.0f && ext_ph < 10.0f){
		ext_midnight_ph = ext_ph;
	}
	if(ext_temperature > 1 && ext_temperature < 50){
		ext_midnight_temperature = ext_temperature;
	}
}

/**
 * Note: This method does more than the name says. That is kind of bad :-(
 * It calculates the pulses since midnight, manages auto-dst clock setting
 * and calls some other functions when a new day starts...
 */
void calculate_pulses_since_midnight(void){
	int32_t new_psm;
	if(speed_factor == 1){
		//Check for changed summer/winter time if neccessary
		uint8_t byte = eeprom_read_byte((uint8_t*)EE_TIME_SETTINGS);
		if(byte&EE_TIME_FLAG_AUTODST){
			//Auto DST is active. Check for status:
			uint8_t dst = TRUE;
			if( rtc.month < 3 || rtc.month > 10 )                 // month 1, 2, 11, 12
				dst = FALSE;
			if( rtc.day - rtc.dow >= 25 && ( !(EE_TIME_FLAG_DST&byte) || rtc.hrs >= 2) ){ // after last Sunday 2:00
				if( rtc.month >= 10 )                           // October -> Winter
					dst = FALSE;
			}else{                                        // before last Sunday 2:00
				if( rtc.month <= 3 )                            // March -> Winter
					dst = FALSE;
			}
			
			//Has it changed?
			if(dst && !(byte&EE_TIME_FLAG_DST) && rtc.hrs < 23){
				//Set DST
				rtc.hrs++;
				ds1302_set_date(rtc);
				byte |= EE_TIME_FLAG_DST;
				eeprom_write_byte((uint8_t*)EE_TIME_SETTINGS, byte);
			}else if(!dst && (byte&EE_TIME_FLAG_DST) && rtc.hrs > 0){
				//Unset DST
				rtc.hrs--;
				ds1302_set_date(rtc);
				byte &= ~EE_TIME_FLAG_DST;
				eeprom_write_byte((uint8_t*)EE_TIME_SETTINGS, byte);
			}
		}
		
		new_psm = ((int32_t)rtc.hrs)*216000L + ((int32_t)rtc.min)*3600L + ((int32_t)rtc.sec)*60L + ((int32_t)sec_part);
		if(new_psm < pulses_since_midnight){
			//Its a new day
			reset_brightness();
		}
		//Set new value
		pulses_since_midnight = new_psm;
	}
}

/**
 * Diese Methode stellt die PWM AusgÃ¤nge ein uns definiert den 30Hz
 * Puls des Systems.
 */
void timer_init(void)
{
	/**
	 * Timer1 wird unsere Systemuhr definieren.
	 * Modus: CTC, Normal Port Operation, Prescaler clk/1024
	 * Freq.: 14745600/(N*(1+OCR0)) = 60.0Hz
	 *        mit OCR0 = 239 und N = 1024
	 */
	TCCR0 = (1 << WGM01) | (1 << CS02) | (1 << CS01) | (1 << CS00);
	OCR0 = 239;
	TIMSK |= (1 << OCIE0);
}

/**
 * Calculates the brightness of every channel for the current time stamp.
 */
void calculateIntensities(void){
	float weight;
	float timeSpan;
	dbEntry dbtemp;
	
	//Calculating intensity values
	
	while(pulses_since_midnight >= next_entry_ptime){
		profile_index++;
		if(profile_index < 128){
			//Swap next to last entry
			last_entry_ptime = next_entry_ptime;
			last_entry_red_brightness = next_entry_red_brightness;
			last_entry_green_brightness = next_entry_green_brightness;
			last_entry_blue_brightness = next_entry_blue_brightness;
			last_entry_white_brightness = next_entry_white_brightness;
			//Use the next entry in the future, if defined
			dbtemp = get_entry(profile_index);
			next_entry_ptime = dbtemp.timestamp*120L;
			next_entry_red_brightness = dbtemp.brightness_red;
			next_entry_green_brightness = dbtemp.brightness_green;
			next_entry_blue_brightness = dbtemp.brightness_blue;
			next_entry_white_brightness = dbtemp.brightness_white;
		}else{
			//Swap next to last entry
			last_entry_ptime = 0;
			last_entry_red_brightness = BRIGHTNESS_RED;
			last_entry_green_brightness = BRIGHTNESS_GREEN;
			last_entry_blue_brightness = BRIGHTNESS_BLUE;
			last_entry_white_brightness = BRIGHTNESS_WHITE;
			//Go around the corner
			profile_index = 0;
			dbtemp = get_entry(profile_index);
			//Fake the timestamp to be the very last entry
			next_entry_ptime = pulses_per_day + dbtemp.timestamp*120L;
			next_entry_red_brightness = dbtemp.brightness_red;
			next_entry_green_brightness = dbtemp.brightness_green;
			next_entry_blue_brightness = dbtemp.brightness_blue;
			next_entry_white_brightness = dbtemp.brightness_white;
		}
		/* Consider moonphase at the blue and green channel
		 * if white channel is still "dark"
		 */
		if(next_entry_white_brightness < 512 && use_moon_phase){
			next_entry_blue_brightness = next_entry_blue_brightness/(float)moon_phase_factor;
			next_entry_green_brightness = next_entry_green_brightness/(float)moon_phase_factor;
		}
	}
	timeSpan = next_entry_ptime - last_entry_ptime;

	weight = ((float)(pulses_since_midnight - last_entry_ptime) / timeSpan);

	BRIGHTNESS_RED = (uint16_t)((last_entry_red_brightness*(1-weight))
					+ ((next_entry_red_brightness)*weight));
	
	BRIGHTNESS_GREEN = (uint16_t)((last_entry_green_brightness*(1-weight))
					+ ((next_entry_green_brightness)*weight));
	
	BRIGHTNESS_BLUE = (uint16_t)((last_entry_blue_brightness*(1-weight))
					+ ((next_entry_blue_brightness)*weight));
	
	BRIGHTNESS_WHITE = (uint16_t)((last_entry_white_brightness*(1-weight))
					+ (next_entry_white_brightness*weight));
					
}

/**
 * Fancy ISR@60Hz
 * Das ist der Puls unseres Systems
 */
ISR(SIG_OUTPUT_COMPARE0){
	pulse_flag = TRUE;
}

/**
 * This makro contains all the calls which are associated with the 
 * compare0 interrupt.
 */
void pulse_makro(void){
	pulse_flag = FALSE; //Unset flag
	
	//Increment system pulse clock
	pulses_since_midnight += speed_factor;
	if(pulses_since_midnight >= pulses_per_day){
		//A new day!
		pulses_since_midnight = 0;
		reset_brightness();
	}
	
	//Reset watchdog
	wdt_reset();

	//Calling the dcf77-driver
	dcf77_pulse();
	
	//Updating intensities
	if(operation){
		if(mode != MODE_NO_CHANNEL_UPDATE){
			calculateIntensities();
		}
	}else{
		//If not on operation mode, shut down LEDs
		BRIGHTNESS_RED = BRIGHTNESS_GREEN = BRIGHTNESS_BLUE = BRIGHTNESS_WHITE = 0;
	}
	
	// Process possible input from keypad and render mainscreen or menuscreen 	
	menu_state = processKeyInput();
	
	//Run buffer engine
	buffer_engine();
	
	//Doing this just once per second
	if(++sec_part == 60){
		sec_part = 0; //Resetting sec_part
		sec_flag = TRUE; //Set second flag
		//Update clock
		rtc = ds1302_get_date();
		calculate_pulses_since_midnight();
		
		if(rtc.sec%30 == 0) {
			//calculate flow rate
			//Moving average filter
			flowmeter_rate = ((float)flowmeter_ticks * flowmeter_liter_per_tick * FLOWMETER_UPSCALE); //ticks_per_minute * l/tick * 6 * 60m
			               //+ (flowmeter_rate * 0.25); 0.015
			/*if (flowmeter_rate > flowmeter_rate_min){
				flowmeter_rate_min = flowmeter_rate;
				//eeprom_write_block((uint16_t*)EE_FLOWMETER_MIN, &flowmeter_rate_min, sizeof(flowmeter_rate_min));
				eeprom_write_word((uint16_t*)EE_FLOWMETER_MIN, (uint16_t)(flowmeter_rate_min*10.1));
				eeprom_busy_wait();
				sys_status_vec &= ~_BV(WARN_EXT_FLOWMETER);
			} 
			else*/ 
			if (flowmeter_rate < flowmeter_rate_min) {
				sys_status_vec |= _BV(WARN_EXT_FLOWMETER);
			} else{
				sys_status_vec &= ~_BV(WARN_EXT_FLOWMETER);
			}
			//Reset tick counter
			flowmeter_last_ticks = flowmeter_ticks;
			flowmeter_ticks = 0;
		}
		
		//Idle time is always raised, it is then resetted on any button input in processKeyInput
		pwm_raise_idle_time();
		if(pwm_backlight_get() != 0 && pwm_backlight_needs_dimming() == TRUE){
			// Dim lcd brightness once a second if neccesary
			pwm_backlight_set((pwm_backlight_get()-1));
		}

		//The mainscreen shows basic informations
		if(menu_state == OUTSIDE_MENU){
			render_mainscreen();  //Select nothing, so give 0xFF
		}
		
		//Acoustical warning if system has problems
		if(!operation && rtc.sec == 0){
			beep_on();
		}else if(rtc.sec == 1){
			beep_off();
		}
	}
}

//Unsere Main
int main (void)
{
	wdt_disable();
	
	//char emergency_stop = FALSE; //By setting this flag, the system can be powered down for ever. DEPRECATED
	uint16_t red_current_error_counter = 0, white_current_error_counter = 0;
	#if (HW_REVISION >= 2)
	uint16_t blue_current_error_counter = 0, green_current_error_counter = 0;
	#endif
	uint16_t voltage_error_counter = 0;
	
	/** Driver Inits **/
	beep_init();
	beep_on();
	
	//Configure ADC
	ADC_init();
	
	//RS232
	//USARTinit();
	
	//Flowmeter
	//PORTE |= _BV(PORTE0);
	//eeprom_read_block(&flowmeter_rate_min, (uint16_t*)EE_FLOWMETER_MIN, sizeof(flowmeter_rate_min));
	flowmeter_rate_min = (float)eeprom_read_word((uint16_t*)EE_FLOWMETER_MIN);
	flowmeter_liter_per_tick = (float)eeprom_read_word((uint16_t*)EE_FLOWMETER_LPK) / 10000.0;
	
	//SPI Init
	spi_init();
	
	//DOGM Display
	lcd_init();
	
	//Getting system teperature for LED calibration
	while(mb_temperature < -272){
		readTemperature(TEMP_MB);
	}
	//Checking the driver currents at startup
	DDRE |= (1<<PORTE5)|(1<<PORTE4)|(1<<PORTE3);; //RGB channels
	DDRB |= (1<<PORTB5); //White channel
	
	//Activating the leds
	PORTE |= (1<<PORTE5)|(1<<PORTE4)|(1<<PORTE3);
	PORTB |= (1<<PORTB5);
	_delay_ms(50); //Wait for the LEDs to really get started
	//measuring
	readInitialLEDCurrent(VOLT_WHITE);
	readInitialLEDCurrent(VOLT_RED);
	#if (HW_REVISION >= 2)
	readInitialLEDCurrent(VOLT_GREEN);
	readInitialLEDCurrent(VOLT_BLUE);
	#endif
	//Deactivating the leds
	PORTE &= ~((1<<PORTE5)|(1<<PORTE4)|(1<<PORTE3));
	PORTB &= ~(1<<PORTB5);
	
	//PWM Konfigurieren
	pwm_init();
	
	//Init buttons
	button_init();
	
	//Initialize DS1302
	ds1302_init();
	
	//DCF77 Initialization
	dcf77_init();
	rtc = ds1302_get_date();
	calculate_pulses_since_midnight();
	
	//Calculate moon-phase and get related settings
	moon_phase = calculate_moon_phase(rtc.year, rtc.month, rtc.day);
	calc_moon_phase_factor();
	use_moon_phase = eeprom_read_byte((uint8_t*)EE_TIME_SETTINGS)&EE_TIME_USE_MOON_PHASE;
	
	//Lower backlight intensity to default value
	pwm_backlight_set(pwm_backlight_get_default());
	
	//Timer initialisieren
	timer_init();
	
	beep_off();
	
	//Show the splash screen
	render_splashscreen();
	
	_delay_ms(1000);
	
	//Set Watchdog to 2s timeout reset
	wdt_enable(WDTO_2S);
	
	
	//Enable global interrupts
	sei();

	//This is absolutely necessary to be compatible with newer GCC versions than 4.2.x
	#define FAULT_THRESHOLD 100
	while(TRUE){
		/* This is our free time - when we don't have any interrupt to process.
		 * We shall used to put a little bit safety into our machine
		 */
		
		//Check for pulse-flag (at different places)
		if(pulse_flag){
			pulse_makro();
		}
		
		//So, let us do system checks
		char checks_passed = TRUE;
		
		//Checking the temperature
		readTemperature(TEMP_MB);
		///Shutdown if Temperatur >70°C
		if((operation && mb_temperature > 70)
		|| (!operation && mb_temperature > 50)){
			checks_passed = FALSE;
			sys_status_vec |= _BV(ERR_MB_TEMPERATUR);
		}else{
			///Restart if temperature is below 50°C again
			sys_status_vec &= ~_BV(ERR_MB_TEMPERATUR);
		}
		
		/*
		 * The LEDs must be active during a current measurement.
		 * We do not try to measure at the very beginning or the end of a pulse.
		 * We need at least 3% duty-cycle to perform measurements.
		 */
		
		if(readLEDCurrent(VOLT_WHITE)){
			if(driver_current_white > driver_current_white_at_startup
			|| driver_current_white > 6.0f){
				white_current_error_counter += 5;
			}else if(white_current_error_counter > 0){
				white_current_error_counter -= 5;
			}
		}
		
		//If the threshold is reached, we must shut down. No need to be too nervous!
		if(white_current_error_counter > FAULT_THRESHOLD){
			checks_passed = FALSE;
			sys_status_vec |= _BV(ERR_WHITE_OVERCURRENT);
			//emergency_stop = TRUE; //We won't resume automatically if problems with the LEDs occure
		}
		
		//Check for pulse-flag (at different places)
		if(pulse_flag){
			pulse_makro();
		}
		
		/*
		 * The LEDs must be active during a current measurement.
		 * We do not try to measure at the very beginning or the end of a pulse.
		 * We need at least 3% duty-cycle to perform measurements.
		 */
		if(readLEDCurrent(VOLT_RED)){
			if(driver_current_red > driver_current_red_at_startup
				|| driver_current_red > 2.0f){
				red_current_error_counter += 5;
			}else if(red_current_error_counter > 0){
				red_current_error_counter -= 5;
			}
		}
		
		//If the threshold is reached, we must shut down. No need to be too nervous!
		if(red_current_error_counter > FAULT_THRESHOLD){
			checks_passed = FALSE;
			sys_status_vec |= _BV(ERR_RED_OVERCURRENT);
			//emergency_stop = TRUE; //We won't resume automatically if problems with the LEDs occure
		}
		
		//Check for pulse-flag (at different places)
		if(pulse_flag){
			pulse_makro();
		}
		
		#if (HW_REVISION >= 2)
		/*
		 * The LEDs must be active during a current measurement.
		 * We do not try to measure at the very beginning or the end of a pulse.
		 * We need at least 3% duty-cycle to perform measurements.
		 */
		if(readLEDCurrent(VOLT_GREEN)){
			if(driver_current_green > driver_current_green_at_startup
				|| driver_current_green > 2.0f){
				green_current_error_counter += 5;
			}else if(green_current_error_counter > 0){
				green_current_error_counter -= 5;
			}
		}
		
		//If the threshold is reached, we must shut down. No need to be too nervous!
		if(green_current_error_counter > FAULT_THRESHOLD){
			checks_passed = FALSE;
			sys_status_vec |= _BV(ERR_GREEN_OVERCURRENT);
			//emergency_stop = TRUE; //We won't resume automatically if problems with the LEDs occure
		}
		
		//Check for pulse-flag (at different places)
		if(pulse_flag){
			pulse_makro();
		}
		
		/*
		 * The LEDs must be active during a current measurement.
		 * We do not try to measure at the very beginning or the end of a pulse.
		 * We need at least 3% duty-cycle to perform measurements.
		 */
		if(readLEDCurrent(VOLT_BLUE)){
			if(driver_current_blue > driver_current_blue_at_startup
				|| driver_current_blue > 2.0f){
				blue_current_error_counter += 5;
			}else if(blue_current_error_counter > 0){
				blue_current_error_counter -= 5;
			}
		}
		
		//If the threshold is reached, we must shut down. No need to be too nervous!
		if(blue_current_error_counter > FAULT_THRESHOLD){
			checks_passed = FALSE;
			sys_status_vec |= _BV(ERR_BLUE_OVERCURRENT);
			//emergency_stop = TRUE; //We won't resume automatically if problems with the LEDs occure
		}
		
		//Check for pulse-flag (at different places)
		if(pulse_flag){
			pulse_makro();
		}
		
		#endif
		
		//Now we have some time to update the voltage measurements
		readVoltage(VOLT_12);
		
		//Check the voltages
		#if (HW_REVISION < 2)
		readVoltage(VOLT_3); //DEPRICATED on HW_REV > 2
		if(((mb_3V3 < 2.9f || mb_3V3 > 3.6f)
			|| (mb_12V < 11.0f || mb_12V > 13.0f))
			&& voltage_error_counter < 1000){
			voltage_error_counter++;
		}else if(voltage_error_counter > 0){
			voltage_error_counter--;
		}
		#else
		if((mb_12V < 11.0f || mb_12V > 13.0f)
			&& voltage_error_counter < 1000){
			voltage_error_counter++;
		}else if(voltage_error_counter > 0){
			voltage_error_counter--;
		}
		#endif
		
		//React on threshold
		if(voltage_error_counter > FAULT_THRESHOLD){
			checks_passed = FALSE;
			sys_status_vec |= _BV(ERR_SUPPLY_VOLTAGE);
		}else{
			sys_status_vec &= ~_BV(ERR_SUPPLY_VOLTAGE);
		}
		
		//Stop or resume operation - no resume if errors on LEDs occured!
		if(checks_passed 
		&& !(sys_status_vec & (_BV(ERR_RED_OVERCURRENT)
							 | _BV(ERR_GREEN_OVERCURRENT)
							 | _BV(ERR_BLUE_OVERCURRENT)
							 | _BV(ERR_WHITE_OVERCURRENT)))){
			operation = TRUE;
		}else{
			operation = FALSE;
		}
		
		//Check for pulse-flag (at different places)
		if(pulse_flag){
			pulse_makro();
		}
		
		//Just casual measurements
		//Slow the measure rate down to avoid 50Hz swing on that kind
		if(sec_flag){
			sec_flag = FALSE;
			readPH(PH);
		}
		readTemperature(TEMP_EXT); ///TODO: SUPPORT SEPERATE CURVE??
		
		if(ext_midnight_ph > 0.0f && 
		  (ext_ph < ext_midnight_ph-0.5f || ext_ph > ext_midnight_ph+0.5f)){
			//Warn PH
			sys_status_vec |= _BV(WARN_EXT_PH);
		}else{
			sys_status_vec &= ~_BV(WARN_EXT_PH);
		}
		
		if(ext_midnight_temperature != 0 && 
		  (ext_temperature < (ext_midnight_temperature - 1) || ext_temperature > (ext_midnight_temperature + 1))){
			sys_status_vec |= _BV(WARN_EXT_TEMPERATURE);
		}else{
			sys_status_vec &= ~_BV(WARN_EXT_TEMPERATURE);
		}
		
		//Checking slow meter input
		if(PINE & _BV(PINE0)) {
			//If input is high
			if(flowmeter_level == 0) {
				flowmeter_ticks++;
				flowmeter_level = 1;
			}
		} else {
			//If input is low
			flowmeter_level = 0;
		}
	}
}
