/***************************************************************************
 *            Prometheus.h
 *
 *  Wed Apr 25 14:55:56 2007
 *  Copyright  2007  Oliver Schwaneberg
 *  Oliver.Schwaneberg@hvbg.de
 ****************************************************************************/

#ifndef _PROMETHEUS_
#define _PROMETHEUS_

#include <avr/io.h>
#include <avr/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <stdint.h>
#include <stdlib.h>
#include <inttypes.h>

/**
 * Every profile has got 128 entries
 */
typedef struct{	 
	uint32_t timestamp;
	uint16_t brightness_red;
	uint16_t brightness_green;
	uint16_t brightness_blue;
	uint16_t brightness_white;
} dbEntry;

#include "Profiles.h"
#include "thirdparty/DOGM/dogm-graphic.h"
#include "thirdparty/DOGM/font.h"
#include "thirdparty/SD/config.h"
#include "thirdparty/SD/file.h"
#include "thirdparty/SD/fat.h"
#include "thirdparty/SD/mmc.h"
#include "thirdparty/mem-check/mem-check.h"
#include "drivers/spi.h"
#include "drivers/DS1302.h"
#include "drivers/ADC.h"
#include "drivers/beeper.h"
#include "drivers/button.h"
#include "drivers/Comm.h"
#include "drivers/database.h"
#include "drivers/eeprom_extra.h"
#include "drivers/PWM.h"
#include "drivers/dcf77.h"
#include "drivers/sensors.h"
#include "Renderer.h"
#include "DisplayBuffer.h"
#include "moonphase.h"
#include "Menu.h"

#define FALSE (0!=0)
#define TRUE  (0==0)

//Berechnet die Leuchtstärke der LEDs neu
void calculateIntensities(void);

extern int32_t pulses_since_midnight;
extern uint16_t speed_factor;
extern uint8_t moon_phase;
extern uint8_t use_moon_phase;
extern date rtc;

//External temperature in celsius
extern int16_t ext_midnight_temperature;

//External ph probe
extern float ext_midnight_ph;

//Flowmeter
extern float flowmeter_rate, flowmeter_rate_min, flowmeter_liter_per_tick;
extern uint16_t flowmeter_last_ticks;
#define FLOWMETER_UPSCALE 120.0

extern uint8_t operation, mode;
extern uint16_t sys_status_vec;

//Status flags for "sys_status_vec"
#define ERR_RED_OVERCURRENT		0
#define ERR_BLUE_OVERCURRENT	1
#define ERR_GREEN_OVERCURRENT	2
#define ERR_WHITE_OVERCURRENT	3
#define ERR_MB_TEMPERATUR		4
#define ERR_SUPPLY_VOLTAGE		5
#define WARN_EXT_TEMPERATURE	6
#define WARN_EXT_PH				7
#define WARN_EXT_FLOWMETER      8

//Flags for "mode"
#define MODE_STANDARD			0
#define MODE_NO_CHANNEL_UPDATE	1

//Pulses since last second
extern uint8_t sec_part;

//Reset brightness settings
void reset_brightness(void);

#endif
