/**
 * Renderer.h
 * Provides high-level methods for Display access.
 */

#ifndef _RENDERER_
#define _RENDERER_

#include "Prometheus.h"

#define EDIT_TIME01_HOUR   0
#define EDIT_TIME02_MINUTE 1
#define EDIT_TIME03_SECOND 2
#define EDIT_TIME04_DAY    3
#define EDIT_TIME05_MONTH  4
#define EDIT_TIME06_YEAR   5
void render_tmp(void);

/*
 * Displays auto-off editor.
 */
void render_editor_auto_off(PGM_P title, PGM_P unit, uint8_t index);

/**
 * Displays a brightness editor.
 */
void render_editor_brightness(PGM_P title,  uint8_t index);


/**
 * Displays a date-editor.
 * Parameters:
 * title - define the title of the editor
 * active_entry must be one of EDIT_TIME01_HOUR
 *                             EDIT_TIME02_MINUTE
 *                             EDIT_TIME03_SECOND
 *                             EDIT_TIME04_DAY
 *                             EDIT_TIME05_MONTH
 *                             EDIT_TIME06_YEAR
 * rtc - is the date which is displayed
 */
void render_editor_date(PGM_P title, uint8_t active_entry, date rtc);

/**
 * Render ph reference points
 */
void render_editor_ph(PGM_P title, uint8_t active_entry, float ref, int16_t value);

/**
 * Set current flow rate
 */
void render_editor_flow(PGM_P title, uint8_t active_grip, uint16_t value);

/**
 * Displays the standard "standbye"-screen.
 * This contains date, time, profile, temperature, ph, brightness ...
 */
void render_mainscreen(void);

/**
 * Displays a summary of system informations.
 */
void render_sysinfos(void);

/**
 * Renders the demo screen
 */
void render_demoscreen(void);

/**
 * Render test screen to manually alter channels
 */
void render_testscreen(uint8_t channel);

/**
 * Displays a menu with a list of entries and a title.
 */
//void render_menu(char title[], char labels[][22], int8_t active_index, int8_t number_of_labels);
void render_menu_B(char title[], PGM_P labels[], int8_t active_index, int8_t number_of_labels);
void render_menu_P(PGM_P title, PGM_P labels[], int8_t active_index, int8_t number_of_labels);
void render_boolean_menu_P(PGM_P title, PGM_P labels[], uint8_t booleans[], int8_t active_index, int8_t number_of_labels);
void render_windowed_browser(char title[], char labels[3][22], uint8_t offset, int8_t active_index, int8_t number_of_labels);

/**
 * Renders credits of the world famous ledrise muffers!
 * BTW: Everything and everybody needs credits!!
 */
void render_credits(void);

/**
 * Renders loading result of a specific LED profile
 */
void render_profile_loading_result(char profile_name[], int8_t loading_result);

/**
 * SPLASH Screen
 */
void render_splashscreen(void);

void render_messagebox(PGM_P title, PGM_P message);

void render_messagebox_progress(PGM_P title, char msg[], int8_t progress);

#endif
