/***************************************************************************
 *            Comm.c
 *
 *  Wed Nov  2 17:06:55 2005
 *  Copyright  2005  Oliver Schwaneberg, FB02.FH-Bonn-Rhein-Sieg, BGIA
 *  oliver.schwaneberg@smail.inf.fh-brs.de
 ****************************************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/delay.h>
#include <avr/pgmspace.h>
#include <compat/deprecated.h>

#include <inttypes.h>
#include <stdlib.h>
#include <stdio.h>

#define TRUE (1==1)
#define FALSE (1!=1)

static char comLOCK = FALSE;

char isCharAvailable(void)
{	
	// Does the RX0 bit of the USART Status and Control Register
	// indicate a char has been received?
	if ( (UCSR0A & (0x80)) ) return 1;
	else return 0;
}

char receiveChar(void)
{
	// Return the char in the UDR0 register
	return UDR0;
}

void sendChar(char data)
{
	if(comLOCK)
		return;
	comLOCK = TRUE;
	
    uint16_t i = 0;

    // To send data with the USART put the data in the USART data register 
    UDR0 = data;

    // Check to see if the global interrupts are enabled
    if(SREG & 0x80)
    {
	// Wait until the byte is sent or we count out
		while ( !(UCSR0A&0x40) && (i<40000) ) //Das war ne 10000, aber wir haben ja 16MHz statt 2 MHz
		{
			asm volatile ("nop"::); //Hier muss quasi der Takt halbiert werden, da 80000 nicht in uint16 passt.
			i++;
		}
    }
    else  // Wait until the byte is sent
        while( !(UCSR0A&0x40) );
        
    	// Clear the TXCflag        
	UCSR0A=UCSR0A|0x40; 

	comLOCK = FALSE;		
}

void sendString(char s[])
{	
	int i = 0;
	
	while(i < 64) // don't get stuck if it is a bad string
	{
		if( s[i] == '\0' ) break; // quit on string terminator
		sendChar(s[i++]);
	}
}


void sendInteger(int value)
{
	//char *buffer= (char*) malloc(sizeof(char)*10);
	char *buffer = "          ";
	
	itoa(value, buffer, 10);
	
	sendString(buffer);
	
	//free(buffer);
}

void sendUnsignedInteger(int value)
{
	char *buffer = "          ";
	
	utoa(value, buffer, 10);
	
	sendString(buffer);
}

void sendFloat(float value){
	char *buffer = "          ";
	
	dtostrf(value, 2, 3, buffer);
	
	sendString(buffer);
}

void USARTinit(void)
{
    // Set the USART baudrate registers for 19200
	int UBRR = 47; //19200 Baud
    UBRR0H = (uint8_t) (UBRR>>8);
    UBRR0L = (uint8_t) (UBRR);

    // Enable 2x speed change
    //UCSRA = (1<<U2X0);
	//UBRR = 103; //19200 Baud
	//UBRR = 51; //38400 Baud
	
    // Enable receiver and transmitter
    UCSR0B = (1<<RXEN)|(1<<TXEN)|(0<<RXCIE)|(0<<UDRIE);

    // Set the USART to asynchronous at 8 bits no parity and 1 stop bit
	UCSR0C = (1 << UCSZ01) | (1 << UCSZ00);
}
