/**

**/
#include "PWM.h"

uint8_t needs_dimming = FALSE;
uint8_t auto_off_timeout = 5;
uint8_t idle_time = 0;

void pwm_init(void){
	//Enabling output
	DDRE |= (1<<PORTE5)|(1<<PORTE4)|(1<<PORTE3);
	DDRB |= (1<<PORTB6)|(1<<PORTB5);
	
	/** 16-Bit Version                                               *
	 * Hier verwenden wir keinen Prescaler, d.h. der Counter lÃ¤uft   *
	 * mit 14.7456MHz. Bedeutet eine Wiederholungsrate von           *
	 * 14.745.600 / 65536 = 225Hz                                    *
	 * Hardware-VerknÃ¼pfungen:                                       *
	 * RED: COM3A, GREEN: COM3B, BLUE: COM3C, WHITE: COM1A           *
	 * LCD Backlight: COM1B                                          */
	ICR1 = 0xFFFF; //Definiere einen festen TOP-Wert fÃ¼r PWM, in diesem Fall 16Bit.
	TCCR1A = (1 << COM1A1) | (1 << COM1B1) | (1 << WGM11);
	TCCR1B = (1 << CS10) | (1 << WGM12) | (1 << WGM13);
	
	ICR3 = 0xFFFF; //Definiere einen festen TOP-Wert fÃ¼r PWM, in diesem Fall 16Bit.
	TCCR3A = (1 << COM3A1) | (1 << COM3B1) | (1 << COM3C1) | (1 << WGM31);
	TCCR3B = (1 << CS30) | (1 << WGM32) | (1 << WGM33);
}

void pwm_set(uint16_t r, uint16_t g, uint16_t b, uint16_t w){
	OCR1A = w;
	OCR3A = r;
	OCR3B = g;
	OCR3C = b;
}


void pwm_backlight_set(uint8_t index){
	if(index >= LCD_BRIGHTNESS_LEVELS){
		index = LCD_BRIGHTNESS_LEVELS - 1;
	}

	if(current_brightness_index <= 0 && index > 0){
		//RE-ENABLE LCD DISPLAY PORT
		TCCR1A = (1 << COM1A1) | (1 << COM1B1) | (1 << WGM11);
	}

	current_brightness_index = index;

	switch (current_brightness_index) {
		case 0:
			OCR1B = 0x0000;
			//SET LCD DISPLAY PORT OFF  TODO Oli: Disabling COMB1 does not turn backlight off :-(
			TCCR1A &= ~(1 << COM1B1);
			PORTB &= ~_BV(PORTB6);
		break;
		case 1:
			OCR1B = 0x1111;
			break;
		case 2:
			OCR1B = 0x4444;
			break;
		case 3:
			OCR1B = 0x4444;
			break;
		case 4:
			OCR1B = 0x8888;
			break;
		case 5:
			OCR1B = 0xBBBB;
			break;
		case 6:
			OCR1B = 0xFFFF;
			break;
	}

	//reset needs_dimming flag
	if(current_brightness_index <= 0){
		needs_dimming = FALSE;
		current_brightness_index = 0;
		OCR1B = 0x0000;
	}

}

void pwm_backlight_set_default(uint8_t index){
	//Store settings
	eeprom_write_byte((uint8_t*)EE_DISPLAY_BRIGHTNESS, index);
}

int8_t pwm_backlight_get(void){
	return current_brightness_index;
}

uint8_t pwm_backlight_get_default(void){
	//Store settings
	uint8_t val = eeprom_read_byte((uint8_t*)EE_DISPLAY_BRIGHTNESS);
	if (val < 7){
		return val;
	}else{
		return 3;
	}
}

void pwm_raise_idle_time(void){
	idle_time++;
}

void pwm_reset_idle_time(void){
	pwm_backlight_set(pwm_backlight_get_default());
	idle_time = 0;
}


uint8_t pwm_backlight_needs_dimming(void){
	//At first check if auto-off time has exceeded
	if(!eeprom_read_byte((uint8_t*)EE_DISPLAY_DIMMING)){
		return FALSE;
	}

	if(idle_time >= auto_off_timeout){
		needs_dimming = TRUE;
	}
	return needs_dimming;
}

uint8_t pwm_get_auto_off_idle_timeout(void){
	return auto_off_timeout;
}
void pwm_set_auto_off_idle_timeout(uint8_t new_timeout_secs){
	auto_off_timeout = new_timeout_secs;
}



