/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/* #include <avr/io.h>
 #include <stdlib.h>*/
 #include "drivers/eeprom_extra.h"
 #include "drivers/database.h"

//Profile 1 should use Byte 92 to 1300 of 4096
#define EE_PROFILE1_NAME     92
#define EE_PROFILE1_START   (EE_PROFILE1_NAME+8)

//Profile 1 should use Byte 1301 to 2508 of 4096
#define EE_PROFILE2_NAME    (EE_PROFILE1_START+1280)
#define EE_PROFILE2_START   (EE_PROFILE2_NAME+8)

//Profile 1 should use Byte 2509 to 3716 of 4096
#define EE_PROFILE3_NAME    (EE_PROFILE2_START+1280)
#define EE_PROFILE3_START   (EE_PROFILE3_NAME+8)

/**
 * This private method provides the start-eeprom address of a 
 * profile with a certain index. Default is 0xFFFF
 */
uint16_t getNameAddr(uint16_t p_index){
	if(p_index == PROFILE_USER_1){
		return EE_PROFILE1_NAME;
	}else if(p_index == PROFILE_USER_2){
		return EE_PROFILE2_NAME;
	}else if(p_index == PROFILE_USER_3){
		return EE_PROFILE3_NAME;
	}else{
		return 0xFFFF;
	}
}

/**
 * This private method provides the start-eeprom address of the first
 * entry of a profile. Default is 0xFFFF
 */
uint16_t getAddr(uint16_t p_index){
	if(p_index == PROFILE_USER_1){
		return EE_PROFILE1_START;
	}else if(p_index==PROFILE_USER_2){
		return EE_PROFILE2_START;
	}else if(p_index==PROFILE_USER_3){
		return EE_PROFILE3_START;
	}else{
		return 0xFFFF;
	}
}

void get_profile_name(uint8_t profile, char name[9]){
	eeprom_read_string(getNameAddr(profile), name);
}

void set_profile_name(uint8_t profile, char name[9]){
	eeprom_write_string(getNameAddr(profile), name);
}

/**
  * Returns a specified entry
  */
dbEntry get_eeprom_entry(uint8_t profile, uint8_t e_index){
	 dbEntry result;
	 
	//If it is a preset inside the PROGMEM
	if(profile >= PROFILE_USER_1 && profile <= PROFILE_USER_3){
		//Its a profile in the EEPROM
		uint16_t addr = getAddr(profile);
		//Applying the offsets
		addr += e_index*10;
		
		result.timestamp = eeprom_read_word((uint16_t*)addr);
		addr += 2;
		result.brightness_red = eeprom_read_word((uint16_t*)addr);
		addr += 2;
		result.brightness_green = eeprom_read_word((uint16_t*)addr);
		addr += 2;
		result.brightness_blue = eeprom_read_word((uint16_t*)addr);
		addr += 2;
		result.brightness_white = eeprom_read_word((uint16_t*)addr);
	 }else{
		result.timestamp = 0;
		result.brightness_red = 0;
		result.brightness_green = 0;
		result.brightness_blue = 0;
		result.brightness_white = 0;
	 }
	 return result;
 }
 	 
 /**
  * Overwrites a certain entry inside the EEPROM
  * 
  * Returns: TRUE if successful
  *          FALSE if index is out of range
  */
uint8_t storeEntry(uint8_t profile, uint8_t index, dbEntry newEntry){
	//If it is a preset inside the PROGMEM
	if(profile >= PROFILE_USER_1 && profile <= PROFILE_USER_3){
		//Its a profile in the EEPROM
		uint16_t addr = getAddr(profile);
		//Applying the offsets
		addr += index*10;
		
		eeprom_write_word((uint16_t*)addr, newEntry.timestamp);
		eeprom_busy_wait();
		addr += 2;
		eeprom_write_word((uint16_t*)addr, newEntry.brightness_red);
		eeprom_busy_wait();
		addr += 2;
		eeprom_write_word((uint16_t*)addr, newEntry.brightness_green);
		eeprom_busy_wait();
		addr += 2;
		eeprom_write_word((uint16_t*)addr, newEntry.brightness_blue);
		eeprom_busy_wait();
		addr += 2;
		eeprom_write_word((uint16_t*)addr, newEntry.brightness_white);
		eeprom_busy_wait();
		return TRUE;
	 }else{
		return FALSE;
	}
}
 
 /**
  * Deletes a profile from the EEPROM
  * 
  * Returns: TRUE if successful
  *          FALSE if index is out of range
  */
uint8_t deleteProfile(uint8_t profile){
	uint16_t addr = getNameAddr(profile);
	 if(addr == 0xFFFF){
		 return FALSE;
	 }
	
	uint16_t end_addr = addr+1288;
	//Now its time for deletion
	while(addr < end_addr){
		eeprom_write_word(&addr, 0xFFFF);
		addr += 2;
	}
	return TRUE;
}
