#include <inttypes.h>
#include <avr/interrupt.h>
#include "drivers/eeprom_extra.h"
#include "drivers/database.h"

/**
 * Limited to a string length of 8 characters
 */
void eeprom_write_string(uint16_t addr, char string[]){
	for(uint8_t i = 0; i<8; i++) // don't get stuck if it is a bad string
	{
		/* Wait for completion of previous write */
		while(EECR & (1<<EEWE));
		/* Set up address and Data Registers */
		EEAR = addr++;
		EEDR = string[i];
		/* Write logical one to EEMWE */
		EECR |= (1<<EEMWE);
		/* Start eeprom write by setting EEWE */
		EECR |= (1<<EEWE);
		/* increment address for next iteration */
		//*addr += 1;
		if( string[i] == '\0' ){
			break; // quit on string terminator
		}
	}
}

char *eeprom_read_string(uint16_t addr, char string[]){
	char temp_char;
	
	/* Wait for completion of previous write */
	while(EECR & (1<<EEWE));
	 
	for(uint8_t i = 0; i<8; i++){
		/* Set up address register */
		EEAR = addr++;
		/* Start eeprom read by writing EERE */
		EECR |= (1<<EERE);
		temp_char = EEDR;	
		string[i] = temp_char;
		if(temp_char == '\0'){
			break;
		}
	}
	string[8]='\0';
	return string;
}
