#ifndef _eeprom_extra_h_
#define _eeprom_extra_h_

#include "Prometheus.h"

/** 
 * Defining a little map for options and settings
 * Note: Byte 92 and above is reserved for profiles!
 *       (See database.h for further informations)
 **/

//Store the CRC16 of the current firmware
#define EE_FW_CRC16          11
//Adresses to store a Firmware Version identifier
#define EE_FW_BUILD          13
//Auto FW-Upgrade
#define EE_AUTO_FW_UPGRADE   15

//One byte language for the settings
#define EE_I18N              16
//One byte for the clock settings (Flags for DCF, AM-PM/24h, DST and auto-DST, moon-phase)

#define EE_TIME_SETTINGS     17
#define EE_TIME_FLAG_DCF     0x80
#define EE_TIME_FLAG_24H     0x40
#define EE_TIME_FLAG_DST     0x20
#define EE_TIME_FLAG_AUTODST 0x10
#define EE_TIME_USE_MOON_PHASE 0x08

//Current profile
#define EE_CURRENT_PROFILE   18

//Display settings
#define EE_DISPLAY_BRIGHTNESS 19
#define EE_DISPLAY_DIMMING    20

//PH probe settings (Reference points and measure results)
#define EE_PH_REF1            22
#define EE_PH_VAL1            24
#define EE_PH_REF2            26
#define EE_PH_VAL2            28

//Mainscreen Settings
#define EE_MAINSCREEN_SLOT1   64
#define EE_MAINSCREEN_SLOT2   65
#define EE_MAINSCREEN_SLOT3   66
#define EE_MAINSCREEN_SLOT4   67

//Flowmeter maximum
#define EE_FLOWMETER_MIN      72
#define EE_FLOWMETER_LPK      74

/**
 * Writes a string of 8 bytes to the eeprom
 */
void eeprom_write_string(uint16_t addr, char string[]);

/**
 * Readdds a string of 8 bytes from the eeprom
 */
char *eeprom_read_string(uint16_t addr, char string[]);

/**
 * Read an dbentry from the eeprom at given start-address
 */
//dbEntry eeprom_read_dbentry(const uint16_t *addr);

/**
 * Write an dbentry to the eeprom
 */
//void eeprom_write_dbentry(const uint16_t *addr, dbEntry dbentry);

#endif
