#ifndef _SENSORS_
#define _SENSORS_

#include <avr/io.h>
#include <avr/pgmspace.h>
#include "../Prometheus.h"

#if (HW_REVISION < 2)
#define VOLT_3      PF0
#define VOLT_5      PF1
#else
#define VOLT_BLUE   PF0
#define VOLT_GREEN  PF1
#endif

#define VOLT_12     PF2
#define TEMP_MB     PF3
#define TEMP_EXT    PF4
#define PH          PF5
#define VOLT_WHITE  PF6
#define VOLT_RED    PF7

//RDS(on) of the used transistors in Ohm @ 30°C
#define RDS_IRF520_RS		0.188f
//Drift per degree celsius
#define RDS_IRF520_DRIFT	0.01f //0.0085f

//Define the number of seconds used for ph oversampling
#define PH_OVERSAMPLING 9 //15
//#define PH_SAMPLES_PER_CALL (uint32_t)10

void readVoltage(char sensor);
char readLEDCurrent(char sensor);
void readInitialLEDCurrent(char sensor);
void readTemperature(char sensor);
void readPH(char sensor);

void initPH(void);

//Mainboard temperature
extern int16_t mb_temperature;
//External temperature in celsius
extern int16_t ext_temperature;
//Supply voltages
extern float mb_12V;
#if (HW_REVISION < 2)
extern float mb_3V3;
extern float driver_current_white, driver_current_red;
extern float driver_current_white_at_startup, driver_current_red_at_startup;
#endif
#if (HW_REVISION >= 2)
extern float driver_current_white, driver_current_red, driver_current_green, driver_current_blue;
extern float driver_current_white_at_startup, driver_current_red_at_startup, driver_current_green_at_startup, driver_current_blue_at_startup;
#endif
//External ph probe
extern float ext_ph;
extern float ext_raw_ph;

#endif
