#define DOUBLESPEED (1<<SPI2X)
#define SINGLESPEED (0<<SPI2X)

#include "drivers/spi.h"

uint8_t spi_prescaler = 0;
uint8_t spi_mode = DOUBLESPEED;

void spi_init(void){
	//Configure ship select lines
	PORTC |= _BV(PC3)|_BV(PC0);
	DDRC  |= _BV(PC3)|_BV(PC0);
  	/* Set MOSI, SCK and ^SS output, all others input */
	DDRB = (1<<PB2)|(1<<PB1)|(1<<PB0);
	//init spi: msb first, update falling, read rising, clk/2=7.3728 MHz
	SPCR = (0 << SPIE) | (1 << SPE) | (0 << DORD) | (1 << MSTR) | (0 << CPOL) | (0 << CPHA) | spi_prescaler;
	SPSR = spi_mode;
	SPDR = 0;
	
	spi_dummywrite();
	spi_wait_for_idle();
}

void spi_dummywrite(void){
	spi_write(0xFF);
	spi_wait_for_idle();
}

void spi_slow_down(void){
	if(spi_mode == DOUBLESPEED){
		spi_mode = SINGLESPEED;
	}else if(spi_prescaler < 3){
		spi_prescaler++;
		spi_mode = DOUBLESPEED;
	}
	SPCR = (0 << SPIE) | (1 << SPE) | (0 << DORD) | (1 << MSTR) | (0 << CPOL) | (0 << CPHA) | spi_prescaler;
	SPSR = spi_mode;
}

void spi_highspeed(void){
	SPCR = (0 << SPIE) | (1 << SPE) | (0 << DORD) | (1 << MSTR) | (0 << CPOL) | (0 << CPHA);
	SPSR = DOUBLESPEED;
}

void spi_restore(void){
	SPCR = (0 << SPIE) | (1 << SPE) | (0 << DORD) | (1 << MSTR) | (0 << CPOL) | (0 << CPHA) | spi_prescaler;
	SPSR = spi_mode;
}

void spi_set_highspeed(void){
	spi_mode = DOUBLESPEED;
	spi_prescaler = 0;
	SPCR = (0 << SPIE) | (1 << SPE) | (0 << DORD) | (1 << MSTR) | (0 << CPOL) | (0 << CPHA) | spi_prescaler;
	SPSR = spi_mode;
}
